package psimjava;
/**
// This class will generate Requests for the system
// PsimJ model.
// (C) Jose M. Garrido, 2001, 2003
// Updated mar 2005
//
*/

abstract public class Arrivals extends psimjava.Process {
    // Input Parameters
    private double closeArrival;

    // References
    private Erand arrPeriodgen; // random inter-arrival period

    // Computed Values
    private static int arrived = 0;

    ////////////////////////////////////////////////////////////////////////////
    /**
     * Requests are generated by the Arrivals Process
     * @param s String - name of the Arrivals process
     * @param sIn Input 0 Input Parameters
     */
    public Arrivals(String s, Input sIn) {
        super(s);
        closeArrival = sIn.getCloseArrivals();

        // create objects for grnrration of random variables
        double arrMean = sIn.getInterArrivalPeriod();

        arrPeriodgen = new Erand(arrMean, 3); // using seed = 3
    }

    ////////////////////////////////////////////////////////////////////////////
    /**
     * The Arrivals process generate Requests using a Poisson distribution
     */
    public void Main_body() {
      try {
        while (get_clock() <= closeArrival) {

          // generate inter-arrival period
          double ttr = arrPeriodgen.fdraw();

          delay(ttr); // wait for Request arrival
          //
          arrived++;
          // Generate new Request
          generateRequest(arrived);
          try {
            Thread.sleep(Model.getAnimationDelay());
          }
          catch (InterruptedException e) {}

        }
        Output.trace("Closing " + get_name());
        terminate();
      }
      catch (Throwable ex) {
        Output.reportError(ex);
      }
    } // end Main body

    ////////////////////////////////////////////////////////////////////////////
    /**
     * method to generate the specific Request for this model
     * @param requestNum int
     */
    abstract protected void generateRequest(int requestNum);

    ////////////////////////////////////////////////////////////////////////////
    /**
     * Return a count of the number of Arrivals tha thave occured
     * @return int - number of arrivals
     */
    public static int getArrived() {
        return arrived;
    }

    ////////////////////////////////////////////////////////////////////////////
    public void reportStatistics() {

    }
} // end class Arrivals
