package psimjava;

/**
 * PsimJ Java package for OO simulation
 * Communication timer.
 * <P> An instance of this process type is used by several processes
 * to cause a timeout when communicating.
 *
 * <P>Object-oriented simulation, process interaction approach.
 * <P> (C) J. M. Garrido. June 2000, 2003, 2004
 *
 * <P> Class for controlling Communication timer (used by most processes when
 *     communicating)
 * <P> File: Comm_timer.java
 *
 */
class Comm_timer
    extends Process {
    private double wait_period; // period to wait for communication (timeout)
    private Waitq comm_chan; // communication channel
    private Process waiting_proc; // waiting process to check for timeout
    private boolean sender_flag; // true if process is sender, otherwise false
    private boolean comm_set; // true if timer set
    //
    /**
     * Constructor for the communication timer object
     * @param proc_comm process waiting to communicate
     * @param wait_p period to wait for timeout
     * @param channel xommunication channel
     * @param send_rec flag with value true if proc_comm is sender, othewise false
     */
    public Comm_timer(Process proc_comm, double wait_p, Waitq channel,
                      boolean send_rec) {
        super("Comm timer");
        waiting_proc = proc_comm;
        wait_period = wait_p;
        comm_chan = channel; // communication channel
        sender_flag = send_rec;
        comm_set = false;
        start();
        // deactivate(this);            // suspend
    }

    //
    public Comm_timer() { // default constructor
        super("Comm timer");
        wait_period = 0.0;
        System.out.println("Comm timer set");
        comm_set = false;
        start();
        // deactivate(this);            // suspend
    }

    //
    /**
     * This function sets the communication timer
     * @param proc_comm process waiting to communicate
     * @param wait_p period to wait for timeout
     * @param channel communication channel
     * @param send_rec flag with value true if proc_comm is sender, othewise false
     */

    public void set_comm_timer(Process proc_comm, double wait_p, Waitq channel,
                               boolean send_rec) {
        waiting_proc = proc_comm;
        wait_period = wait_p;
        comm_chan = channel; // communication channel
        sender_flag = send_rec;
        reactivate(this); //
    }

    //
    /**
     * Main body for a communication timer object
     * does not need to be redefined in user model.
     */
    public void Main_body() {
        Process ppp = null;
        if (!comm_set) {
            deactivate(this);
        }
        while (get_clock() < get_simul_p()) {
            // case START:
            // starting timer
            System.out.println("Comm timer waiting at: " + get_clock());
            // try {
            delay(wait_period); // timeout period
            // }
            // catch (InterruptedException e)
            // { // do nothing
            // }
            // case COMPLETE:
            // check if process is waiting to communicate
            if (waiting_proc.comm_wait()) {
                // System.out.println("Comm timer: " + waiting_proc.t_name
                //      + " waiting to comm");
                if (sender_flag) { // sender
                    System.out.println("Comm timer Interrupting " +
                                       waiting_proc.get_name());
                    if (waiting_proc.idle()) {
                        ppp = null;
                        while (ppp == null) {
                            ppp = comm_chan.coopt();
                        }
                        reactivate(ppp);
                    }
                    // interrupt sender process
                    waiting_proc.p_interrupt(Pobject.INT_COMM_TIMER); // interrupt
                }
                else { // receiver
                    // phase = WAIT;
                    if (waiting_proc.idle()) {
                        comm_chan.qwait();
                    }
                    waiting_proc.p_interrupt(Pobject.INT_COMM_TIMER); // interrupt
                    // terminate();     // terminate
                }
                deactivate(this);
            }
            else { // process is not waiting to communicate
                deactivate(this); // terminate
            }
            // case WAIT:
            //   waiting_proc.p_interrupt(10);   // interrupt for timeout
            //   deactivate(this);
        } // end while
    }
}
//
