package psimjava;

/**
 * PsimJ Java package for OO simulation
 * Class for controlling conditional waiting.
 * <P> Processes that are waiting for a particular condition are
 * enqueued in a condition queue by priority.
 * <P>Object-oriented simulation package, process interaction approach.
 * <P> (C) J. M. Garrido. June 2000, 2003, 2004
 */
class Condq {
    private String title; // title for object of this class
    private boolean all; // flag for testing all proceses in cqueue
    private Pqueue cqueue; // condition priority queue//
    //
    //
    // The condition will normally be a function of type boolean
    // with no parameters
    //
    //
    /**
     * Constructor InitializeS Condq object with title and maximum number of
     * priorities to use.
     * @param name title for the Condq object
     * @param maxprio maximum number of priorities to use
     */
    public Condq(String name, int maxprio) { // constructor
        cqueue = new Pqueue(name, maxprio);
        all = false;
        title = name;
    }

    ////////////////////////////////////////////////////////////////////////////
    /**
     * The process evaluates the condition 'cond'
     * if it evaluates to false, the process is passivated and enqueued
     * @param cond condition to be evaluated.
     */
    public boolean waituntil(boolean cond) {
        Process ps = (Process) StaticSync.this_process();
        Process lp;
        boolean empty = true;
        int i = 0;
        int prio = ps.get_prio();
        if (cond == true) {
            // look for a same or higher priority nonempty queue
            while (empty && i < prio) { // only higher priority 3-28-00
                if ( (cqueue.plength(i)) > 0) {
                    empty = false;
                }
                else {
                    i++;
                }
            }
            if (empty == false) {
                cqueue.into(ps);
                // process is suspended because other process(es)
                // of higher priorities are waiting
                ps.deactivate(ps);
                return false;
            }
            else {
                // continue and
                // promote next process at head of queue
                if (cqueue.length() > 0) {
                    lp = (Process) cqueue.out();
                    lp.reactivate(lp); // activate
                }
                return true;
            }
        }
        else {
            // condition is false
            cqueue.into(ps); // enqueue
            ps.deactivate(ps); // passivate
            // cout << "Process suspended by " << title << "\n";
            return false;
        }
    }

    ////////////////////////////////////////////////////////////////////////////
    /**
     * Unblock (dequeue) the process at the head of the queue, and activate it
     * when the processes become current, it will test the condition
     */
    public void signal() {
        Process lp;
        if (cqueue.length() == 0) {
            return; // cond queue is empty
        }
        else {
            if (all == false) {
                lp = (Process) cqueue.out(); // dequeue process at head
                lp.reactivate(lp); // activate
            }
            else { // dequeue all processes
                while (cqueue.length() > 0) {
                    lp = (Process) cqueue.out();
                    lp.reactivate(lp);
                }
            }
        }
        return;
    }
    ////////////////////////////////////////////////////////////////////////////
    /**
     * set flag, if true test condition in all waiting processes
     *
     */
    public void setall(boolean flag) {
        all = flag; // to test all waiting processes
    }

    ////////////////////////////////////////////////////////////////////////////
    /**
     * Returns length of the condition queue (number of processes waiting)
     */
    public int length() {
        return cqueue.length();
    }
} // end of class condq
//
