package psimjava;

// SimDisplay
// This class implements all outputs of the simulation run
// including the graphic animation, and plotter
// M.E. Shaw Updated 3/9/01
// Modified by Jose M. Garrido 12/9/04
// Modified by R Schlesinger 2/25/05
//
import javax.swing.*;
import java.util.*;
/**
 *
 * <p>Basic Display class for PSIM Modeling Framework</p>
 *
 * <p>Description: You can create an on-screen display of the Models activity
 * by creating a subclass of the Display class. The Frames of the on-screen
 * display will typically be plots of some variables over time and possibly
 * an animation of the models activity. The Display class will automatically
 * cause the frames of the on-screen display to be periodically updated.</p>
 *
 * <p>Copyright: Copyright (c) 2005</p>
 * @author Rich Schlesinger
 * @version 1.0
 */

public class Display
    extends Thread {
    private double simPeriod;

    private static final int ANIMATION_DELAY = 500;

    private ArrayList<JFrame> frames = new ArrayList<JFrame> ();
    ////////////////////////////////////////////////////////////////////////////
    /**
     * Create the screen Displays
     * @param simIn Input - Input parameters
     */
    public Display(Input simIn) {
        simPeriod = simIn.getSimPeriod();

        // Tell Framework that we are doing Animation
        Model.enableAnimation();

        // Start running the animation
        setName("Displays");
        start();

    }

    ////////////////////////////////////////////////////////////////////////////
    /**
     * Add a Frame to the screen display at the specified coordinates
     * @param x int x coordinate
     * @param y int y coordinate
     * @param frame JFrame
     */
    public void add(int x, int y, JFrame frame) {
        frames.add(frame);
        frame.setLocation(x, y);
        frame.setVisible(true);
    }

    ////////////////////////////////////////////////////////////////////////////
    /**
     * Thread to periodically update the animation
     */
    public void run() {
        double prevClock = 0.0;
        try {
            while (prevClock < simPeriod) {
                // Repaint each frame that has been added to the display
                for (JFrame frame : frames) {
                    frame.repaint();
                }
                //Sleep so animation does not run too fast
                prevClock = Model.getCurrentClock();
                try {
                    Thread.currentThread().sleep(ANIMATION_DELAY);
                }
                catch (InterruptedException e) {
                    // If we are interrupted, then stop updating display
                    return;
                }
            }

        }
        catch (Throwable e) {
            Output.reportError(e);
        }
    }
}
