package psimjava;

import java.awt.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.BorderFactory;
import java.util.*;
import java.text.*;

/**
 * <p> Class for specifyng an Input parameter of type double</p>
 *
 * <p>Description: Class for representing an integer simulation parameter</p>
 *
 * <p>Copyright: Copyright (c) 2005</p>
 * @author Rich Schlesinger
 * @version 1.0
 */
public class DoubleParameter extends Parameter implements ChangeListener {
    private double min;
    private double max;
    private double value;
    private double range;

    private static final int sliderMin = 0;
    private static final int sliderMax = 100;
    private static final int sliderRange = sliderMax - sliderMin;

    private static final NumberFormat labelFormat = new DecimalFormat("0.00");
    ////////////////////////////////////////////////////////////////////////////
    /**
     * Integer parameters have a name, a minimum value, maximum value, and default
     * @param name String
     * @param min double
     * @param max double
     * @param value double
     */
    public DoubleParameter(String name, double min, double max, double value) {
        super(name);
        if ((min <= value) & (value <= max)) {
            this.min = min;
            this.max = max;
            this.value = value;
            range = (max - min);
        }
        else {
            JOptionPane.showConfirmDialog(null,
                                          "Parameter \"" + name +
                                          "\" must be min<=value<=max",
                                          "ERROR",
                                          JOptionPane.OK_OPTION,
                                          JOptionPane.ERROR_MESSAGE);
            System.exit( -1);
        }
    }

    ////////////////////////////////////////////////////////////////////////////
    /**
     * Get the current value of the parameter
     * @return double
     */
    public double getValue() {
        return value;
    }
    ////////////////////////////////////////////////////////////////////////////
    /**
     * Get the current value as a String
     * @return String
     */
    protected String getStringValue() {
        return Double.toString(value);
    }
    ////////////////////////////////////////////////////////////////////////////
    /**
     * method used by base class to construct the GUI.Uses a Slider for the selection
     * @return JComponent
     */
    protected JComponent buildSelector() {
        ChangeListener pthis;
        Hashtable labels = new Hashtable<Integer,String>();
        for (int i=sliderMin; i<=sliderMax; i=i+(sliderMax-sliderMin)/10) {
            labels.put(i,new JLabel(labelFormat.format(valueFromSlider(i)).toString()));
         }

        JSlider slider = new JSlider(JSlider.VERTICAL,
                                     sliderMin,
                                     sliderMax,
                                     sliderFromValue(value));
        slider.setLabelTable(labels);
        slider.setBorder(BorderFactory.createLineBorder(Color.black));
        slider.setMajorTickSpacing(sliderMax / 10);
        slider.setMinorTickSpacing(Math.max(1, sliderMax / 100));
        slider.setPaintTicks(true);
        slider.setPaintLabels(true);
        slider.setSnapToTicks(false);
        pthis = (ChangeListener) this;
        slider.addChangeListener(pthis);
        return slider;
    }
   ////////////////////////////////////////////////////////////////////////////
    /**
     * Called by Swing when the slider changes
     * @param e ChangeEvent
     */
    public void stateChanged(ChangeEvent e) {
        value = valueFromSlider(( (JSlider) e.getSource()).getValue());
        displaySelection(Double.toString(value));
    }
    ////////////////////////////////////////////////////////////////////////////
    private double valueFromSlider(int s) {
       return range*(s-sliderMin)/sliderRange + min;
    }
    ////////////////////////////////////////////////////////////////////////////
    private int sliderFromValue(double v) {
        return (int)(sliderRange*(v-min)/range + sliderMin);
    }

}
