package psimjava;

/**
 * <p> First-Come-First-Serve Schedular</p>
 *
 * <p>Copyright: Copyright (c) 2004-2005</p>
 *
 * @author Rich Schlesinger
 * @version 1.0
 */
public class FCFSschedular extends Schedular {

    /**
     * Create First-Come First-Serve schedular with an Unbounded queue
     */
    public FCFSschedular() {
        super("FCFS");
    }

    ////////////////////////////////////////////////////////////////////////////
    /**
     * Create First-Come First-Serve schedular with a queue of the specified size
     * @param qsize - int size of queue for this schedular
     */
    public FCFSschedular(int qsize) {
        super("FCFS", qsize);
    }

    ////////////////////////////////////////////////////////////////////////////
    /**
     * Create First-Come First-Serve schedular with a queue of the specified size
     * @param name String - name of the Schedular
     * @param qsize int - size of the queue for this schedular
     */
    public FCFSschedular(String name, int qsize) {
        super(name, qsize);
    }
    ////////////////////////////////////////////////////////////////////////////
    /**
     * Create First-Come First-Serve schedular with a queue of the specified size
     * @param name String - name of the Schedular
     * @param qsize int - size of the queue for this schedular
     * @param maxUsage int - maximum number of conccurrent users of scheduled object
     */
    public FCFSschedular(String name, int qsize, int maxUsage) {
        super(name, qsize);
        this.maxConcurrentRequests = maxUsage;
    }
    ////////////////////////////////////////////////////////////////////////////
    /**
     * Return next request to be processed.
     * Simply get the first request of the queue
     * @return DiskRequest to be processed (or null if none pending)
     */
    public synchronized Request getNextRequest() {
        // Return immediately if no pending requests
        // or the resource is currently held with exclusive access
        // or the max number of concurrent requests are already being processed
        if ((requests.size()==0) | !isAccessAllowed() ) {
            return null;
        }

        // If currently handling a shared request, we can handle another one
        if (currentRequestType == Request.SHARED_ACCESS) {
            // Loop thru pending requests, looking for another shared access request
            for (Request rqst : requests) {
                if (rqst.getRequestType() == Request.SHARED_ACCESS) {
                    requests.remove(rqst);
                    updateSchedularInfo(rqst);
                    return rqst;
                }
            }
            // none found, we're done
            return null;
        }

        // Otherwise, we are not currently handling any requests and can take first one
        else {
            Request rqst = requests.removeFirst();
            // Indicate we are processing this request
            updateSchedularInfo(rqst);
            return rqst;
        }
    }


}
