package psimjava;

/**
* <p> This class will generate Requests for the system </p>
* (C) Jose M. Garrido, 2001, 2003
* Updated mar 2005
*
*/

abstract public class IncomingRequestGenerator
    extends psimjava.Process {
    // Input Parameters
    private double closeArrival;

    // References
    private Erand arrPeriodgen; // random inter-arrival period

    // Computed Values
    private int arrived = 0;

    ////////////////////////////////////////////////////////////////////////////
    /**
     * Requests are generated by the Arrivals Process
     * @param s String - name of the Arrivals process
     * @param sIn Input 0 Input Parameters
     */

    public IncomingRequestGenerator(String s, Input sIn) {
        super(s);
        closeArrival = sIn.getCloseArrivals();

        // create objects for grnrration of random variables
        double arrMean = sIn.getInterArrivalPeriod();

        arrPeriodgen = new Erand(arrMean, 3); // using seed = 3
    }
    ////////////////////////////////////////////////////////////////////////////
    /**
     * Requests are generated by the Arrivals Process
     * @param s String - name of the Arrivals process
     * @param closeArrival double - time to stop arrivals
     * @param arrMean double - mean arrival time
     */

    public IncomingRequestGenerator(String s, double closeArrival, double arrMean) {
        super(s);
        this.closeArrival = closeArrival;

        arrPeriodgen = new Erand(arrMean, 3); // using seed = 3
    }
    ////////////////////////////////////////////////////////////////////////////
    /**
     * The Arrivals process generate Requests using a Poisson distribution
     */
    public void Main_body() {
        try {
            while (get_clock() <= closeArrival) {

                // generate inter-arrival period
                double ttr = arrPeriodgen.fdraw();

                delay(ttr); // wait for Request arrival
                //
                arrived++;
                // Generate new Request
                generateRequest(arrived);

                Model.screenUpdate();
            }
            Output.trace(get_clock(),"Closing " + get_name());
            terminate();
        }
        catch (Throwable ex) {
            Output.reportError(ex);
        }
    } // end Main body

    ////////////////////////////////////////////////////////////////////////////
    /**
     * method to generate the specific Request for this model
     * @param requestNum int
     */
    abstract protected void generateRequest(int requestNum);

    ////////////////////////////////////////////////////////////////////////////
    /**
     * Return a count of the number of Arrivals tha thave occured
     * @return int - number of arrivals
     */
    public int getArrived() {
        return arrived;
    }

    ////////////////////////////////////////////////////////////////////////////
    /**
     * Report any Incoming Request statistics
     */
    public void reportStatistics() {

    }
} // end class Arrivals
