package psimjava;

import javax.swing.*;
import java.lang.reflect.*;

/**
 * <p> Class to work with the Input Parameters</p>
 * <p>Copyright: Copyright (c) 2005</p>
 *
 * @author Rich Schlesinger
 * @version 1.0
 */
public class Input {
    static protected boolean DISPLAY_SUPPORTED;
    Parameter[] parameters;
    public Input() {
    }

    /**
     * Return an array of the parameters for this simulation
     * Uses reflection to find parameters defined in sublcass
     * @return Parameter[]
     */
    public Parameter[] getParameters() {
        int pIndex = 0;
        // Get the subclass
        Class c = this.getClass();
        // Get list of fields defined in the subclass
        Field[] fields = c.getFields();
        // Create array to hold the parameters
        parameters = new Parameter[fields.length];
        // Scan thru the fields
        for (Field f : fields) {
            try {
                // Get current instance of this field
                Object p = f.get(this);
                if (p instanceof Parameter) {
                    //Add to array if it is a Parameter
                    parameters[pIndex] = (Parameter) p;
                    pIndex++;
                }
            }
            catch (IllegalAccessException e) {}
        }
        return parameters;
    }

    ////////////////////////////////////////////////////////////////////////////
    /**
     * Get Array of Strings with name & value of each Parameter
     * @return String[]
     */
    public String[] getFormattedInputParameters() {
        String[] parms = new String[parameters.length];
        for (int i = 0; i < parameters.length; i++) {
            parms[i] = parameters[i].getName() + ": " +
                parameters[i].getStringValue();
        }
        return parms;
    }

    ////////////////////////////////////////////////////////////////////////////
    /**
     * Returns the Simulation Period
     * @return double
     */
    public double getSimPeriod() {
        return getParameterValue("Simulation Period");
    }

    ////////////////////////////////////////////////////////////////////////////
    /**
     * Return the time at which Arrivals should stop
     * @return double
     */
    public double getCloseArrivals() {
        return getParameterValue("Arrivals Stop");
    }

    ////////////////////////////////////////////////////////////////////////////
    /**
     * Get the inter-arrival period for incoming Requests
     * @return double
     */
    public double getInterArrivalPeriod() {
        return getParameterValue("Inter Arrival Time");
    }

    ////////////////////////////////////////////////////////////////////////////
    /**
     * Get the value of a named Input Parameter
     * @param name String - name of Parameter
     * @return double - value of Parameter
     */
    public double getParameterValue(String name) {
        for (Parameter p : parameters) {
            if (p.getName().equals(name)) {
                return ( (IntegerParameter) p).getValue();
            }
        }
        JOptionPane.showConfirmDialog(null,
                                      "Parameter \"" + name +
                                      "\" does not exist",
                                      "ERROR",
                                      JOptionPane.OK_OPTION,
                                      JOptionPane.ERROR_MESSAGE);
        System.exit( -1);
        return 0.0;
    }
}
