package psimjava;

import java.awt.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.BorderFactory;

/**
 * <p> Class for declaring an Integer Input Parameter</p>
 *
 * <p>Description: Class for representing an integer simulation parameter</p>
 *
 * <p>Copyright: Copyright (c) 2005</p>
 * @author Rich Schlesinger
 * @version 1.0
 */
public class IntegerParameter extends Parameter implements ChangeListener {
    private int min;
    private int max;
    private int value;

    ////////////////////////////////////////////////////////////////////////////
    /**
     * Integer parameters have a name, a minimum value, maximum value, and default
     * @param name String
     * @param min int
     * @param max int
     * @param value int
     */
    public IntegerParameter(String name, int min, int max, int value) {
        super(name);
        if ((min <= value) & (value <= max)) {
            this.min = min;
            this.max = max;
            this.value = value;
        }
        else {
            JOptionPane.showConfirmDialog(null,
                                          "Parameter \"" + name +
                                          "\" must be min<=value<=max",
                                          "ERROR",
                                          JOptionPane.OK_OPTION,
                                          JOptionPane.ERROR_MESSAGE);
            System.exit( -1);
        }

    }

    ////////////////////////////////////////////////////////////////////////////
    /**
     * Get the current value of the parameter
     * @return int
     */
    public int getValue() {
        return value;
    }
    ////////////////////////////////////////////////////////////////////////////
    /**
     * Get the current value as a String
     * @return String
     */
    protected String getStringValue() {
        return Integer.toString(value);
    }
    ////////////////////////////////////////////////////////////////////////////
    /**
     * method used by base class to construct the GUI.Uses a Slider for the selection
     * @return JComponent
     */
    protected JComponent buildSelector() {
        JSlider slider = new JSlider(JSlider.VERTICAL, min, max, value);
        slider.setBorder(BorderFactory.createLineBorder(Color.black));
        slider.setMajorTickSpacing(max / 10);
        slider.setMinorTickSpacing(Math.max(1, max / 50));
        slider.setPaintTicks(true);
        slider.setPaintLabels(true);
        slider.setSnapToTicks(true);
        slider.addChangeListener(this);
        return slider;
    }

    ////////////////////////////////////////////////////////////////////////////
    /**
     * Called by Swing when the slider changes
     * @param e ChangeEvent
     */
    public void stateChanged(ChangeEvent e) {
        value = ( (JSlider) e.getSource()).getValue();
        displaySelection(Integer.toString(value));
    }

}
