   package psimjava;
   import java.util.Random;
/**
*
* <p> Normal Distribution random number generator </p>
* PsimJ Java package for OO simulation
* <p>Object-oriented simulation package, process interaction approach.</p>
* <p> (C) J. M. Garrido. June 2000, 2003, 2004</p>
* Random number generator, Normal distribution
*/

public class Normal extends Random implements RandomGenerator{
// Normal distribution
//
  private long x;
  private long s;
  private double fx;
  private double fs;
  private static double z2 = 0.0;
  
// -------------------------------------------------------------------
 /**
 * Constructor for random number generator object - Normal dist
 * @param mean long the mean value for the distribution
 * @param stdev long the value of the standard deviation
 * @param seed int the value to set the random stream.
 */
 public Normal(long mean, long stdev, int seed) {
   super(seed);
   x = mean;
   s = stdev;
   if (s < 0)
      s = 1;
 }
 /**
 * Constructor for random number generator object - Normal dist
 * @param mean long the mean value for the distribution
 * @param stdev long the value of the standard deviation
 */
 public Normal(long mean, long stdev) {
   super();
   x = mean;
   s = stdev;
   if (s < 0)
      s = 1;
 }
 /**
  * Constructor using values of type double
  * @param rmean double - Mean of the distribution
  * @param rstd double - Standard Deviation of the distribution
  * @param seed int - seed for the distribution
  */
 public Normal(double rmean, double rstd, int seed) {
   super(seed);
   fx = rmean;
   fs = rstd;
   if (fs < 0.0d)
      fs = 1.0;
 }
 /**
  * Constructor using values of type double
  * @param rmean double - Mean of the distribution
  * @param rstd double - Standard Deviation of the distribution
  */
 public Normal(double rmean, double rstd) {
   super();
   fx = rmean;
   fs = rstd;
   if (fs < 0.0d)
      fs = 1.0;
 }

 /**
 * This function generates a random number of type 'long'
 * from the Normal distribution set up by the constructor
 * @return long, a random number.
 */
 public long draw () {
 
    /* 'normal' returns a pseudo-random variate from a normal   */
    /* distribution with mean x and standard deviation s.          */
    double v1;
    double v2;
    double w;
    double z1;
    if (z2 != 0.0)
       {z1 = z2; z2 = 0.0;}  /* use value from previous call */
    else
      {  do
         {
            v1 = 2.0 * (nextDouble()) - 1.0;
            v2 = 2.0 * (nextDouble()) - 1.0;
            w = v1*v1+v2*v2;
         }
         while (w >= 1.0 || w == 0.0);
         // System.out.println( "normal: w = " + w);
         w = Math.sqrt(-2.0 * Math.log ( w ) / w );
         z1 = v1 * w;
         z2 = v2 * w;
      }
     // System.out.println("Value of z1: " + z1 + " z2: " + z2);
     v1 =  (double) x +  (z1 * (double) s );
     // System.out.println( "Normal:return value = " + v1);
     return ((long) v1);
 }
 //
 //
 //
 /**
 * This function generates a random number from Normal distribution
 * set up by the constructor
 * @return double, a pseudo-random number.
 */
 /*
 public double fdraw ()
 {
    return (fx + (nextGaussian() * fs));
 }
 */
  public double fdraw () {
 
    /* 'normal' returns a pseudo-random variate from a normal   */
    /* distribution with mean fx and standard deviation fs.          */
    double v1;
    double v2;
    double w;
    double z1;
    if (z2 != 0.0)
       {z1 = z2; z2 = 0.0;}  /* use value from previous call */
    else
      {  do
         {
            v1 = 2.0 * (nextDouble()) - 1.0;
            v2 = 2.0 * (nextDouble()) - 1.0;
            w = v1*v1+v2*v2;
         }
         while (w >= 1.0 || w == 0.0);
         //System.out.println( "normal: w = " + w);
         w = Math.sqrt(-2.0 * Math.log ( w ) / w );
         z1 = v1 * w;
         z2 = v2 * w;
      }
      return (fx + (z1 * fs));
 }
} // end class Normal
//
