package psimjava;

import java.util.*;
import javax.swing.*;
import java.awt.Color;

/**
 * <p> Base class to represent a parameter for the simulation.
 * Subclasses available for numeric Parameters and Selections</p>
 *
 * <p>Copyright: Copyright (c) 2005</p>

 * @author Rich Schlesinger
 * @version 1.0
 */
public abstract class Parameter {
    private String name;
    private String namePart1 = "";
    private String namePart2 = "";
    private Box guiBox;
    private JLabel label1 = new JLabel(" ");
    private JLabel label2 = new JLabel(" ");
    private JLabel label3 = new JLabel(" ");
    ////////////////////////////////////////////////////////////////////////////
    /**
     * Constructor: All Parameters have a name
     * @param name String
     */
    public Parameter(String name) {
        this.name = name;
    }

    ////////////////////////////////////////////////////////////////////////////
    /**
     * Get the Parameter's name
     * @return String
     */
    public String getName() {
        return name;
    }

    ////////////////////////////////////////////////////////////////////////////
    /**
     * Get a GUI that allows the user to specify the value of the Parameter
     * @return JComponent
     */
    public JComponent getGUI() {
        if (guiBox == null) {
            guiBox = Box.createVerticalBox();

            JComponent selector = buildSelector();
            selector.setAlignmentX(guiBox.LEFT_ALIGNMENT);

            wrapName();
            label1.setText(namePart1);
            label1.setHorizontalAlignment(SwingConstants.CENTER);

            label2.setText(namePart2);
            label3.setText(getStringValue());
            label3.setForeground(Color.MAGENTA);

            guiBox.setAlignmentX(guiBox.LEFT_ALIGNMENT);
            guiBox.setAlignmentY(guiBox.TOP_ALIGNMENT);

            guiBox.add(label1);
            guiBox.add(label2);
            guiBox.add(label3);
            guiBox.add(selector);
        }
        return guiBox;
    }

    ////////////////////////////////////////////////////////////////////////////
    /**
     * Used by subclasses to display current value fo the Parameter
     * @param value String
     */
    protected void displaySelection(String value) {
        label3.setText(value);
    }

    ////////////////////////////////////////////////////////////////////////////
    // This method breaks the name into 2 lines
    private void wrapName() {
        StringTokenizer st = new StringTokenizer(name);
        String[] words = new String[st.countTokens()];
        for (int i = 0; i < words.length; i++) {
            words[i] = st.nextToken();
        }
        int i = 0;
        while (i < words.length - i - 1) {
            namePart1 = (i==0 ? "" : namePart1 + ' ') + words[i];
            namePart2 = words[words.length - i - 1] + (i==0 ? "" : ' ' + namePart2);
            i++;
        }
        if (words.length % 2 == 1) {
            if (namePart1.length()>= namePart2.length()) {
                namePart2 = words[i] + (i == 0 ? "" : ' ' + namePart2);
            }
            else {
                namePart1 = (i==0 ? "" : namePart1 + ' ') + words[i];
            }
        }
        if (namePart1.length()==0) {
            namePart1 = " ";
        }
    }

    ////////////////////////////////////////////////////////////////////////////
    /**
     * Method in subclass to build the selection portion of the GUI
     * @return JComponent
     */
    abstract protected JComponent buildSelector();

    /**
     * Method in subclass to get the current value as a String
     * @return String
     */
    abstract protected String getStringValue();
}
