package psimjava;
import java.util.Random;
/**
 *
 * <p> Poisson Distribution random number generator </p>
 * PsimJ Java package for OO simulation
 * <P>Object-oriented simulation package, process interaction approach.
 * <P> (C) J. M. Garrido. June 2000, 2003, 2004
 * <P>Object-oriented simulation package, process interaction approach
 */

public class Poisson extends Random implements RandomGenerator {
    //
    private double rate;
    private double period;
    /**
     * Constructor to initialize a random number generator using a Poisson
     * distribution.
     * @param prate double, the rate of arrival (arrivals per unit time).
     * @param pperiod double, period considered for arrivals.
     * @param seed long, the seed for selecting an appropriate random stream.
     */
    public Poisson(double prate, double pperiod, long seed) {
        super(seed);
        rate = prate;
        period = pperiod;
    }

    /**
     * Constructor to initialize a random number generator using a Poisson
     * distribution.
     * @param prate double, the rate of arrival (arrivals per unit time).
     * @param pperiod double, period considered for arrivals.
     */

    public Poisson(double prate, double pperiod) {
        super();
        rate = prate;
        period = pperiod;
    }

    /**
     * This function generates a random number using a Poisson distribution.
     * @return long, a random number that represents the number of arrivals.
     */
    public long draw() {
        double x = 1.0;
        double thresh;
        long n = 0;
        thresh = Math.exp( - (rate * period));
        x = nextDouble();
        while (x >= thresh) {
            x *= nextDouble();
            n++;
        }
        return n;
    }
}
//
