package psimjava;

/**
 * PsimJ Java package for OO simulation
 * <P>Object-oriented simulation package, process interaction approach.
 * PsimJ (C)Jose M. Garrido, June 2000, August 2003
 * <P> (C) J. M. Garrido. June 2000, 2003, 2004
 *
 * Priority queue class for PsimJ.
 * <P> PsimJ package, process interaction approach to simulation.
 * (C) J. M. Garrido, June 2000, August 2003, 2004
 */
public class Pqueue {
    private String qname; // title for queue
    private Squeue prqueue[]; // Pobject.MAX_PRIORITIES
    private int qprio; // max value of priority to use
    //
    /**
     * Initializes priority queue object with title and maximum
     * priority to use.
     */
    public Pqueue(String a_name, int max_prio) { // constructor
        String s_name;
        qname = new String(a_name); // create new string
        prqueue = new Squeue[max_prio];
        qprio = max_prio - 1;
        for (int i = 0; i <= qprio; i++) {
            s_name = new String(a_name + Integer.toString(i));
            prqueue[i] = new Squeue(s_name);
        }
    }

    // overloaded constructor
    /**
     * Initializes priority queue object with title, maximum number of priorities
     * to use, and the capacity of the component queues.
     * @param s_name title for the priority queue
     * @param max_prio maximum priority to use
     * @param maxsize capacity (maximum size) of the component queues
     */
    public Pqueue(String a_name, int max_prio, int maxsize) { // constructor
        String s_name;
        qname = new String(a_name); // create new string
        prqueue = new Squeue[max_prio];
        qprio = max_prio - 1;
        for (int i = 0; i <= qprio; i++) {
            s_name = new String(a_name + Integer.toString(i));
            prqueue[i] = new Squeue(s_name, maxsize);
        }
    }

    //
    /**
     * Returns the total length of the priority queue
     */
    public int length() {
        // length of complete queue
        int ssize = 0;
        for (int i = 0; i <= qprio; i++) {
            ssize = ssize + prqueue[i].length();
        }
        return ssize;
    }

    //
    /**
     * Returns a true if total queue is empty,
     * otherwise, return false
     */
    public boolean empty() {
        int ssize = 0;
        for (int i = 0; i <= qprio; i++) {
            ssize = ssize + prqueue[i].length();
        }
        return (ssize == 0);
    }

    //
    /**
     * Returns the length of a particular queue with given priority
     * @param prio priority of the queue
     */
    public int plength(int prio) {
        return prqueue[prio].length();
    }

    //
    /**
     * Returns true if queue is full of processes with priority prio,
     * returns false otherwise
     */
    public boolean full(int prio) {
        return prqueue[prio].full();
    }

    //
    /**
     * Return the maximum priority used
     */
    public int maxprio() {
        return qprio;
    }

    //
    /**
     * Enqueue the specified object (process) into the priority queue
     * @param ps The object (process) to insert into the queue
     */
    public void into(Object ps) {
        Process proc = (Process) ps;
        // System.out.println("Pqueue: into "+proc.get_name());
        // enqueue ps into appropriate queue (priority of ps)
        int prio; // priority of ps
        Pobject pobj_ps = (Pobject) ps;
        prio = pobj_ps.get_prio();
        prqueue[prio].into(ps);
    }

    //
    /**
     * Remove the specified object (process) from the priority queue
     * @param ps The object (process) to remove from the queue
     */
    public void premov(Object ps) {
        // remove ps from queue prio
        int prio; // priority of ps
        Pobject pobj_ps = (Pobject) ps;
        prio = pobj_ps.get_prio();
        prqueue[prio].remov(ps);
    }

    //
    /**
     * Place the specified object (process) at the head of the queue
     * @param ps The object (process) to place into the queue
     */
    public void pback(Object ps) {
        // put ps back at head pf queue prio
        int prio; // priority of ps
        Pobject pobj_ps = (Pobject) ps;
        prio = pobj_ps.get_prio();
        prqueue[prio].back(ps);
    }

    //
    /**
     * Dequeue object (process) with the highest priority
     * @returns Returns an object (process)
     */
    public Object out() {
        // get highest priority object
        Object ps = null;
        boolean empty = true;
        int i = 0;
        while (empty && i <= qprio) {
            if (prqueue[i].length() > 0) {
                empty = false;
                ps = prqueue[i].out();
            }
            i++;
        }
        return ps;
    }

    //
    /**
     * Dequeues an object (process) with the specified priority
     * @param prio Priority of the object to dequeue
     * @returns Returns an object (process)from the queue
     */
    public Object pout(int prio) {
        // dequeue object from queue prio
        return prqueue[prio].out();
    }

    //
    /**
     * Get last object enqueued with the specified priority
     * @param prio Priority of the object to retrieve
     * @returns Returns an object (process) from the priority queue
     */
    public Object plast(int prio) {
        // get last object enqueued on queue prio
        return prqueue[prio].last();
    }

    //
    /**
     * get last object enqueued to lowest priority queue
     * @returns Returns an object (process) from the priority queue
     */
    public Object llast() {
        // get last object enqueued to lowest priority queue
        int i = qprio;
        Object ps = new Object();
        boolean empty = true;
        while (empty && i >= 0) {
            if (prqueue[i].length() > 0) {
                empty = false;
                ps = prqueue[i].last();
            }
            else {
                i--;
            }
        }
        if (empty == false) {
            return ps;
        }
        else {
            return null;
        }
    }

    //
    /**
     * Get last object enqueued into lowest priority queue, down to the
     * specified priority
     */
    public Object pllast(int pr) {
        // get last object enqueued to lowest priority queue, up tp pr
        int i = qprio;
        Object ps = new Object();
        boolean empty = true;
        while (empty && i > pr) {
            if (prqueue[i].length() > 0) {
                empty = false;
                ps = prqueue[i].last();
            }
            else {
                i--;
            }
        }
        if (empty == true) {
            return null;
        }
        else {
            return ps;
        }
    }

} // end of Pqueue
