package psimjava;

/**
 * Basic behavior of a process in PsimJ.
 * <P>The user's Java classes for processes must inherit this class.
 * <P> PsimJ, a java package for OO discrete-event simulation.
 * (C) Jose M. Garrido, June 2000, August 2003, Oct 2004
 */
public abstract class Process
    extends Sched {
    private int interrupt_level; // interrupt level
    private boolean comm_flag; // true if proc waits to comm
    private String t_name; // process name
    private double remaining_t; // rem time when interrupted
    private static double simul_p; // simulation period
    // private static Startlock stlock;
    Suspend susobj; // Sync for suspend/reactivate
    //
    /**
     * Constructor for processes. It initializes the process to an appropriate
     * state.
     * @param pname String, the name to assigned to the process.
     */
    protected Process(String pname) {
        super();
        t_name = pname;
        setName(pname);
        // System.out.println ("Creating " + t_name);
        interrupt_level = 0; // set Psim int
        set_state(IDLE);
        susobj = new Suspend();
        // stlock = new Startlock();
        StaticSync.incr_created();

    } // end constructor

    /////////////////////////////////////////////////////////////////////////////////////////
    /**
    * This method overides the start method in the Java Thread class
    * It inserts the Process into the out queue and then starts the thread running
    * It must be called to have the Process actually start running
    */
    public void start() {
        StaticSync.insert(this, 0.0);
        super.start();
    }

    /**
     *
     * Gets the name of the calling process (activate object).
     * @return String Returns the String with the name of the process
     */
    public String get_name() {
        return t_name;
    }

    //
    // --------------------------------------------------------------
    private void resultis(double val) {
        Thread pthis = (Pobject) Thread.currentThread();
        Pobject p = StaticSync.this_process();
        // System.out.println("resultis (Process) pthis: "+pthis.getName()+
        //    " Running: "+p.getName());
        cancel(val);
        if (p == (Pobject) pthis) {
            StaticSync.schedule();
        }
        // System.out.println("Exiting resultis for: "+pthis.getName());
        return;
    } // end resultis

    //
    //
    // --------------------------------------------------------------
    /**
     * Terminates the called process (object).
     * Change the state of the process to TERMINATED, and the time of
     * the next event to zero.
     */
    public void terminate() {
        cancel(0.0);
        // Thread pthis = (Pobject) Thread.currentThread();
        Process pthis = (Process) StaticSync.this_process();
        Process ppp = this;
        // System.out.println("Terminate - executing thread: "+ pthis.get_name());
        // System.out.println("Terminating process " + get_name());
        StaticSync.removerc(this); // remove from run chain
        set_state(TERMINATED);
        if (this == pthis) {
            StaticSync.schedule();
            pthis = (Process) StaticSync.this_process();
            pthis.susobj.proceed();
        }
        // System.out.println("Terminating process to suswait " + get_name());
        StaticSync.place_term(this); // place in the terminated proc list
        /*
               try {
          susobj.set_flag(true);
          susobj.suswait();  // Java wait/notify mechanism
          // Thread.sleep(600000);
               }
               catch (InterruptedException x) { }
         */
    } // end terminate

    //
    /**
     * Checks if the calling process has been terminated.
     * @return boolean Returns 'true' if the process has been terminated,
     * otherwise returns 'false'.
     */
    public boolean is_terminated() {
        // System.out.println("Testing " + mproc.get_name() + " for termination");
        // Process pthis = (Process) Thread.currentThread();
        // if (! pthis.isAlive())
        //   return false;
        if (rdstate() == TERMINATED) {
            return true;
        }
        else {
            return false;
        }
    }

    //
    //
    // -------------------------------------------------------------
    /**
     * Deactivates (i.e.) suspends the called process.
     * The process will remain suspended until another process reactivates it.
     * @param p Process, the process to suspend.
     */
    public void deactivate(Process p) {
        /* If the process executing this method is deactivating itself,
         then it must call the scheduler.

               // Thread pthis = (Pobject) Thread.currentThread();
               Pobject ppp = (Pobject) StaticSync.this_process();
               // System.out.println("deactivate " + p.getName()+" "+p.get_name()+ " pthis: "+
               //          pthis.getName()+" p_running: "+ppp.getName());
               while ((Pobject) StaticSync.this_process() !=
                                   (Pobject) Thread.currentThread()) {
         //System.out.println("deactivate: p_running: "+
         //   StaticSync.this_process().getName()+
         //   " pthis: "+ pthis.getName() + " yield");
         Thread.yield();     // yield to another process
               }
         if (p.rdstate() == Sched.RUNNING)                // is this process active?
          p.remove();
               //System.out.println(p.getName()+": Idle status now "+p.idle());
         StaticSync.place_sus(p);   // place in list of suspended processes
               if ( p == (Process) ppp ) {  // is process deactivating itself?
         StaticSync.schedule();
               }
               try {
         susobj.set_flag(true);
         StaticSync.susobj.suswait(this);
               } catch ( InterruptedException x) {
          x.printStackTrace();
               }
               // System.out.println("Exiting deactivate of: "+p.getName());
         */
        StaticSync.deactivate(p);
    }

    //
    // -----------------------------------------------------------------
    /**
     * Reactivates the called process.
     * The called process becomes activate again to be scheduled and carry
     * out normal activities.
     * @param p Process, the process to reactivate.
     */
    public void reactivate(Process p) {
        // Thread pthis = (Pobject) Thread.currentThread();
        // Pobject ppp = (Pobject) StaticSync.this_process();
        // System.out.println("Reactivate "+p.getName()+" " + p.get_name()+
        //     " pthis: "+pthis.getName()+
        //       " running: "+ppp.getName());
        // while ((Pobject) Thread.currentThread() !=
        //           (Pobject) StaticSync.this_process()) {
        //   System.out.println("Reactivate, yield");
        //   Thread.yield();
        //}
        StaticSync.reactivate(p);
    }

    //
    // --------------------------------------------------------------
    private void deactivate2() { // suspend
        // this process is delayed until the end of the simulation period
        // when reactivated, it must be interrupted
        long jt;
        if (rdstate() == RUNNING) { // is this process active?
            StaticSync.removerc(this);
        }
        //if ( p_running == (Pobject) Thread.currentThread() ) {
        //   p_running = null;
        StaticSync.schedule();
        jt = (long) (simul_p - StaticSync.get_clock()) * 1000;
        try {
            Thread.sleep(jt);
        }
        catch (InterruptedException x) {
            // try {
            delay(0);
            return; // right now
            // }
            ///catch (InterruptedException e) {
            //   }
        }
    } // end sleep

    // -------------------------------------------------------------------
    /**
     * Schedules the calling process at 'd' time units from the current clock time
     * In the mean time, the process remains active in the event list waiting
     * for the time to become the currently running process.
     * @param d double, the interval from the current simulation time after which the
     * current process will become the rurring process.
     */
    public void delay(double d) {
        // throws InterruptedException  {
        boolean same = false;
        // This implementation uses wait/notify mechanism in Java
        Pobject pthis = (Pobject) Thread.currentThread();
        Process pppp = (Process) StaticSync.this_process();
        // System.out.println(get_name()+" delay for " + d+" running: "+pppp.get_name());
        // System.out.println ("(delay) p_running: " + pppp.getName());
        //while ((Pobject) StaticSync.this_process() !=
        //        (Pobject) Thread.currentThread() ) {
        //    System.out.println("delay yield, pthis: "+pthis.getName());
        //    Thread.yield();
        //}
        // if (this == StaticSync.pfirst)  {
        //   Thread.yield();
        //  int j = 35;
        //  while ( j >= 0) {
        //    Thread.yield();
        //    j--;
        //  }
        // }
        // insert process reference in the event list at appropriate simulation time
        StaticSync.insert( (Sched)this, d);
        if (this == pppp) { // is process delaying itself ?
            StaticSync.schedule(); // get process from run chain
            // System.out.println(get_name() + " In delay() after calling schedule() ");
            pppp = (Process) StaticSync.this_process();
            // System.out.println("delay: this: " + this.get_name() + " new running: " +
            //    pppp.get_name());

            if (this == pppp) { // new running is the same process
                same = true;
            }
            else {
                pppp.proceed(); // Java wait/notify mechanism
            }
        }
        //

        // System.out.println("(Process)delay of " + get_name() +
        //      " before wait/notify block ");
        pppp = (Process) StaticSync.this_process();
        if (same != true && pppp != this) {
            try {
                susobj.set_flag(true);
                susobj.suswait(); // Java wait/notify mechanism
            }
            catch (InterruptedException x) {}
        }
        //
        // System.out.print(get_name()+ " exiting delay ");
        Thread.yield();
        StaticSync.show_rchain(); // used for debuging
    } // end delay

    //
    // ----------------------------------------------------------
    /**
     * Interrupts the calling process using the specified interrupt level.
     * The interrupted process is rescheduled immediately so it can take
     * appropriate action.
     * Note that this is different from the Java Thread interrupt.
     * @param int_level int, the interrupt level used with the interrupt.
     */
    public void p_interrupt(int int_level) {
        // this is the thread interrupted by p_running
        Process pthis = (Process) StaticSync.this_process();
        // System.out.println("p_interrupt "+this.get_name() + " by " +
        //      pthis.get_name());
        if (rdstate() != RUNNING) { // state of process is RUNNING ?
            System.out.println("Proc to interrupt " + get_name() +
                               " is not running");
            process_error(E_TASKPRE);
        }
        //
        remaining_t = rdtime() - StaticSync.get_clock();
        // if (rdstate() == RUNNING)  {
        //    StaticSync.removerc(this);
        // }
        // else
        interrupt_level = int_level;
        StaticSync.insert( (Sched)this, 0.0); // reschedule for now
        // delay(0.0);
        //
        // System.out.println("Exiting p_interrupt");
    } // end p_interrupt

    //
    /**
     * Get the remaining interval left after the process has been interrupted.
     * @return double the period left of the interrupted process
     * (a value of type double).
     */
    public double get_remain_t() {
        return remaining_t;
    }

    // -----------------------------------------------------------
    /**
     * Internal method used by Psim.
     */
    void preempt() {
        // p is the thread to interrupt, not the current thread
        if (rdstate() == RUNNING) {
            StaticSync.removerc(this);
        }
        else {
            process_error(E_TASKPRE);
        }
        remaining_t = rdtime() - StaticSync.get_clock();
        interrupt();
    }

    //
    // -----------------------------------------------------------
    /**
     * Schedules the called process at time 'timeat'. At that instant, the
     * process will become the running process.
     * @param timeat double, the instant at which the calling process will be scheduled.
     */
    public void schedat(double timeat) {
        double simclock = StaticSync.get_clock();
        if (timeat < simclock) {
            process_error(E_SCHTIME);
        }
        // try {
        delay(timeat - simclock);
        // }
        // catch (InterruptedException e) { }
    }

    //
    /**
     * This method sets up and starts executing the main body of the
     * process. This method is called by the 'start' method in the Java Thread
     * package.
     */
    public void run() {
        Process pprun;
        pprun = (Process) StaticSync.this_process();
        /*
               if (pprun != null)
          System.out.println("(Process) to start: " + this.get_name() +
              " running: " + pprun.get_name());
         */
        start_th();
        //
        /*
               pprun = (Process) StaticSync.this_process();
               while (pprun != this) {
               Thread.yield();
               pprun = (Process) StaticSync.this_process();
               }
               //
               // System.out.println("run: "+get_name()+ " running: "+pprun.get_name());
               if (StaticSync.pfirst != null && StaticSync.pfirst != this &&
               pprun != this) {
          // this if also checks if this is the running process
          // if yes, this should not suspend
          // System.out.println(" first: "+StaticSync.pfirst.get_name());
         // System.out.println("run: suspending "+ get_name());
          susobj.set_flag(true);
          try {
              susobj.suswait();           // pthis
          }
          catch (InterruptedException x) {
            // ignore
          }
               }
         */
        // System.out.println("run: invoking purge ");
        // StaticSync.purge(); // flush terminated processes
        //    for (int jjj = 0; jjj < 35; jjj++)
        //       Thread.yield();
        //
        // System.out.println("(process) Starting Main_body of: " +
        //      get_name() + " at: " + get_clock());
        Main_body(); // execute main logic of task
        // System.out.println("Completed Main_body");
        return;
    }

    //
    private void start_th() {
        // Internal method called by run()
        //
        Process ps = this;
        // pthis = (Process) Thread.currentThread();
        // System.out.println("start_th with " + get_name());
        StaticSync.incr_started(); // number of started processes
        // System.out.println("Started: "+StaticSync.get_started()+
        //        " num created: "+ StaticSync.get_created() );
        //
        if (StaticSync.first_process()) { // first process?
            StaticSync.set_first(); // false
            // System.out.println ("(Startlock) First process: " + get_name());
            StaticSync.running(ps); // make current process
            // set_state(Sched.RUNNING);
            StaticSync.pfirst = ps; // the first process (Application thread)
            // StaticSync.insert(ps, 0.0); // insert to front of run chain
        }
        else {
            // System.out.println("(Startlock) inserting: " + get_name());
            // StaticSync.insert(ps, 0.0); // insert to front of run chain
            //
            if (StaticSync.this_process() != this) {
                // System.out.println("start_th; suspending "+get_name());
                susobj.set_flag(true);
                try {
                    susobj.suswait(); // pthis
                }
                catch (InterruptedException x) {
                    // ignore
                }
            } // end if
        }
        // ps.setPriority(4);
        // System.out.println ("Completed setting up " + ps.get_name());
    } // end of start_th

    //
    /**
     * This method represents the main code of every user process. It normally
     * contains the more specialized behavior of the user process. Therefore every
     * user process must implement (override) this method.
     */
    protected abstract void Main_body();

    //
    /**
     * Clears the Psim interrupt level of the process (to zero).
     * Note that this is different than the Java Thread interrupt mechanism.
     */
    public void clear_int() {
        boolean v;
        // v = Thread.interrupted();
        interrupt_level = 0;
    }

    //
    /**
     * Gets the interrupt level of the calling process. This value is zero if the
     * process has not been interrupted, or the flag has been cleared.
     * @return int returns an integer value that represents the
     * interrupt level of the process.
     */
    public int int_level() {
        return interrupt_level;
    }

    //
    //
    /**
     * Checks if the process is idle (i.e., has been suspended).
     * @return boolean Returns true if the process is idle,
     * otherwise returns false.
     */
    public boolean idle() {
        if (rdstate() == IDLE) {
            return true;
        }
        else {
            return false;
        }
    }

    /**
     * Gets the value of the communication flag of the calling process.
     * @return boolean Returns the value of the communication flag, true if the process
     * is waiting to communicate
     */
    public boolean comm_wait() {
        return comm_flag;
    }

    /**
     * Sets the communication flag to the specified truth value
     * @param flag boolean, the boolean value to set the communication flag
     */
    public void set_comm_flag(boolean flag) {
        comm_flag = flag;
    }

    //
    // --------------------------------------------------------

    static void set_simul_p(double tt) {
        simul_p = tt;
    }


    static double get_simul_p() {
        return simul_p;
    }

    //
    //
    private void jjstart_th() {
        // Internal method called by run()
        //
        StaticSync.jjstart_th();
    }

    // ---------------------------------------------------------------
    // check if the number of started processes is the same as the number of
    // created processes.
    // If it is not, return control to the first process.
    //
    private void check_started() {
        StaticSync.incr_started(); // number of started processes
        // System.out.println ("Started: "+ StaticSync.get_started()+
        //    " Created: "+ StaticSync.get_created() );
        if (StaticSync.get_started() < StaticSync.get_created()) {
            StaticSync.insert(this, 0.0);
            // StaticSync.running(StaticSync.pfirst); // first process is running
            // System.out.println(get_name() + " In check_started ");
            // Process pppp = (Process) StaticSync.this_process();
            // System.out.println("check_started this: " + this.get_name() +
            //   " new running: " + pppp.get_name());
            // StaticSync.susobj.proceed();  // Java wait/notify mechanism
            //
            // System.out.println("(Process) " + get_name() +
            //     " before wait/notify block ");
            try {
                susobj.set_flag(true);
                susobj.suswait(); // Java wait/notify mechanism
            }
            catch (InterruptedException x) {}
            //
            //
        }
    } // end check_started

    //
    void proceed() {
        int ind;
        susobj.proceed();
        for (ind = 1; ind < 10; ind++) {
            yield();
        }
    }
} // end Process
