package psimjava;

import java.awt.*;

/**
 * <p>Abstract class for creating a graphic of a Queue</p>
 *
 * <p>Copyright: Copyright (c) 2005</p>
 *
 * @author Rich Schlesinger
 * @version 1.0
 */
abstract public class QGraphic {
    /**
     * Width of queue holder unit
     */
    public static final int QW = 15; // width of queue holder unit
    /**
     * Height of queue holder position
     */
    public static final int QH = 25; // height of queue holder

    // Queue Graphic Orientations
    /**
     * Position queue image horizontally with head at left
     */
    public static final Dimension LEFT = new Dimension(QW, 0);
    /**
     * Position queue image horizontally with head at right
     */
    public static final Dimension RIGHT = new Dimension( -QW, 0);
    /**
     * Position queue image vertically with head at top
     */
    public static final Dimension TOP = new Dimension(0, QH);
    /**
     * Position queue image vertically with head at botton
     */
    public static final Dimension BOTTOM = new Dimension(0, -QH);

    private int qsize; // maximum capacity of queue (default is 20)
    private int jqlength; // current length of queue
    private int xBase;
    private int yBase;
    private int xIncrement;
    private int yIncrement;
    // (x,y) is the coordinate of the far right end of queue, the head
    ////////////////////////////////////////////////////////////////////////////
    /**
     * Create a Graphic at the specified coordinates with the specified orientation
     * @param x int
     * @param y int
     * @param orientation Dimension
     */
    public QGraphic(int x, int y, Dimension orientation) {
        xBase = x;
        yBase = y;
        qsize = 15; // qs
        xIncrement = orientation.width;
        yIncrement = orientation.height;
    }

    ////////////////////////////////////////////////////////////////////////////
    /**
     * Get the current length of the queue
     * @return int
     */
    protected abstract int getQLength();

    ////////////////////////////////////////////////////////////////////////////
    /**
     * Called periodically to update the graphic image
     * @param g Graphics
     */
    public void paint(Graphics g) {
        try {
            // coordinates of head

            int x = xBase;
            int y = yBase;
            jqlength = getQLength();

            for (int i = 0; i < qsize; i++) {
                g.setColor(Color.black);
                g.drawRect(x, y, QW, QH);
                if (i < jqlength) {
                    g.setColor(Color.blue);
                    g.fillOval(x, y + QH / 2 - QW / 2, QW, QW);
                }
                x = x + xIncrement;
                y = y + yIncrement;
            }

            // Draw leading lines
            g.setColor(Color.black);
            g.drawLine(x, y, x + Math.abs(xIncrement), y + Math.abs(yIncrement));
            int xDelta = (xIncrement == 0) ? QW : 0;
            int yDelta = (yIncrement == 0) ? QH : 0;
            g.drawLine(x + xDelta, y + yDelta, x + xDelta + Math.abs(xIncrement),
                       y + yDelta + Math.abs(yIncrement));
        }
        catch (Throwable ex) {
            Output.reportError(ex);
        }

    }
}
