package psimjava;

/**
 * This is an auxiliary class used by class Res.
 * <P> Priority queue class for resource class.
 * <P>Object-oriented simulation package, process interaction approach.
 * (C) Jose M. Garrido, June 2000, August 2003, 2004
 */
class Rqueue {
    private Rsqueue prqueue[]; // Pobject.MAX_PRIORITIES
    private int qprio; // max value of priority to use
    int numres; // number of resources held
    // by current dequeued process
    int nrreq; // number of resources requested
    //
    //
    public Rqueue(int max_prio) { // constructor
        prqueue = new Rsqueue[max_prio];
        qprio = max_prio - 1;
        for (int i = 0; i <= qprio; i++) {
            prqueue[i] = new Rsqueue();
        }
    }

    public int cardinal() {
        // length of complete queue
        int ssize = 0;
        for (int i = 0; i <= qprio; i++) {
            ssize = ssize + prqueue[i].length();
        }
        return ssize;
    }

    //
    public int pcardinal(int prio) {
        return prqueue[prio].length();
    }

    //
    public int maxprio() {
        return qprio;
    }

    public void enqueue(Process lpr, int nr, int nreq) {
        // enqueue ps into appropriate queue (priority of ps)
        int prio; // priority of ps
        prio = lpr.get_prio();
        prqueue[prio].into(lpr, nr, nreq);
    }

    public void premov(Object ps) {
        // remove ps from queue prio
        int prio; // priority of ps
        Pobject pobj_ps = (Pobject) ps;
        prio = pobj_ps.get_prio();
        prqueue[prio].remov(ps);
    }

    public void pputback(Object ps, int nr, int nreq) {
        // put ps back at head pf queue prio
        Qobject lo;
        int prio; // priority of ps
        Pobject pobj_ps = (Pobject) ps;
        prio = pobj_ps.get_prio();
        prqueue[prio].back(ps, nr, nreq);
    }

    public Object dequeue() {
        // get highest priority object
        Qobject lo;
        Object ps = null;
        boolean empty = true;
        int i = 0;
        while (empty && i <= qprio) {
            if (prqueue[i].length() > 0) {
                empty = false;
                lo = prqueue[i].out();
                ps = lo.myobj;
            }
            i++;
        }
        return ps;
    }

    public Object pdequeue(int prio) {
        // dequeue object from queue prio
        Qobject lo;
        Process lp;
        if (prqueue[prio].length() == 0) {
            return null;
        }
        lo = prqueue[prio].out();
        lp = (Process) lo.myobj;
        return lp;
    }

    //
    // Update number of resources requested by process pp
    // called by acquire when there are no sufficient resources to allocate
    public int up_res_req(Process pp, int nr) {
        boolean first = true;
        Qobject lo; // local object
        Process lp; // local process
        int size; // size of queue
        int prio;
        boolean notfound = true;
        prio = pp.get_prio(); // get priority of process pp
        size = (prqueue[prio]).length();
        while (size > 0 && notfound) {
            lo = (Qobject) ( (prqueue[prio]).out());
            if (first == true) {
                lo = (Qobject) ( (prqueue[prio]).show_first());
                first = false;
            }
            else {
                lo = (Qobject) ( (prqueue[prio]).show_next());
            }
            if (lo == null) {
                System.out.println("Object ref is NULL: line 107 Rqueue");
            }
            lp = (Process) (lo.myobj);
            if (lp == pp) {
                notfound = false;
                // get number of resource objects requested
                nrreq = lo.nr_req + nr;
                numres = lo.num_res;
                lo.nr_req = nrreq; // update resources requested
                // queue->prqueue[prio]->putback(lo);
                return nrreq;
            }
            else { // not found
                // prqueue[prio].put(lo); // enqueue again
                size--;
            }
        }
        if (notfound) {
            // add object and resources requested to queue
            // with number of resources held = 0
            lo = new Qobject( (Pobject) pp, 0, nr);
            prqueue[prio].into(lo, 0, nr);
            numres = 0;
            nrreq = nr;
            return nr;
        }
        return 0;
    }

    //
    //
    // get number of resources held by process pp
    public int get_res_h(Process pp) {
        boolean first = true;
        Qobject lo; // local object
        Process lp; // local process
        int size; // size of queue
        int prio;
        boolean notfound = true;
        prio = pp.get_prio();
        size = (prqueue[prio]).length();
        if (size == 0) {
            return 0;
        }
        while (size > 0 && notfound) {
            if (first == true) {
                lo = (Qobject) ( (prqueue[prio]).show_first());
                first = false;
            }
            else {
                lo = (Qobject) ( (prqueue[prio]).show_next());
            }
            lp = (Process) (lo.myobj);
            if (lp == pp) {
                notfound = false;
                // get number of resource objects held by process pp
                numres = lo.num_res;
                // get number of resources requested by process pp
                nrreq = lo.nr_req;
                // return to head of queue
                // queue->prqueue[prio]->putback(lo);
                return numres;
            }
            else { // not found
                // prqueue[prio].into(lo); // enqueue again
                size--;
            }
        }
        if (notfound) {
            numres = 0;
            nrreq = 0;
            return 0; // no resources held
        }
        return 0;
    }

    //
    // Update resources held on release
    public int up_res_rel(Process pp, int nr) {
        boolean first = true;
        int prio;
        Qobject lo; // object with Oqueue, res held and res req
        Process lp;
        int size;
        boolean notfound = true;
        prio = pp.get_prio();
        size = prqueue[prio].length();
        if (size == 0) {
            return 0;
        }
        // search in queue for process pp
        while (size > 0 && notfound) {
            if (first == true) {
                lo = prqueue[prio].show_first(); // just get a copy of first process
                first = false;
            }
            else {
                lo = prqueue[prio].show_next();
            }
            lp = (Process) lo.myobj; // Process component
            if (lp == pp) {
                notfound = false;
                // get number of resources held and requested
                numres = lo.num_res;
                nrreq = lo.nr_req;
                if (numres < nr) {
                    Pobject.process_error(Pobject.E_RESREL);
                }
                if (numres == nr) { // object with no res (held) left?
                    prqueue[prio].rem_cur(); // remove node with object lp
                    numres = 0;
                    nrreq = 0;
                    return 0; // no res held
                }
                if (numres > nr) {
                    numres = numres - nr;
                    lo.num_res = numres;
                    nrreq = 0;
                    lo.nr_req = nrreq;
                    return numres;
                }
            }
            else {
                size--;
            }
        } // end while
        return 0;
    }

    //
    public int up_res_acq(Process pp, int nr) {
        // update the number of resources held on acquire
        boolean first = true;
        Qobject lo;
        Process lp;
        int size;
        int prio;
        boolean notfound = true;
        prio = pp.get_prio();
        size = prqueue[prio].length();
        while (size > 0 && notfound) {
            if (first == true) {
                lo = prqueue[prio].show_first();
                first = false;
            }
            else {
                lo = prqueue[prio].show_next();
            }
            lp = (psimjava.Process) lo.myobj; // lo
            if (lp == pp) {
                notfound = false;
                numres = lo.num_res + nr; // more resources
                lo.num_res = numres;
                lo.nr_req = 0;
                nrreq = lo.nr_req;
                return numres;
            }
            else {
                size--;
            }
        }
        if (notfound) {
            // add object and resources acquired to queue
            prqueue[prio].into(pp, nr, 0);
            numres = nr;
            nrreq = 0;
            return nr;
        }
        return nr;
    }

    //
    //
    // Get number of resources requested by process pp
    // The process does not leave the queue
    public int get_res_req(Process pp) {
        boolean first = true;
        Qobject lo; // local object
        Process lp; // local process
        int size; // size of queue
        int prio;
        boolean notfound = true;
        prio = pp.get_prio();
        size = (prqueue[prio]).length();
        if (size == 0) {
            return 0;
        }
        while (size > 0 && notfound) {
            if (first == true) {
                lo = (Qobject) ( (prqueue[prio]).show_first());
                first = false;
            }
            else {
                lo = (Qobject) ( (prqueue[prio]).show_next());
            }
            lp = (Process) (lo.myobj);
            if (lp == pp) {
                notfound = false;
                // get number of resource objects requested
                nrreq = lo.nr_req;
                // get number of resources held
                numres = lo.num_res;
                // return to head of queue
                // queue->prqueue[prio]->putback(lo);
                return nrreq;
            }
            else { // not found
                // queue->prqueue[prio]->put(lo); // enqueue again
                size--;
            }
        }
        if (notfound == true) {
            nrreq = 0;
            numres = 0;
            return 0; // no requested resources
        }
        return 0;
    }

    //
    //
    public Object plast(int prio) {
        // get last object enqueued on queue prio
        return prqueue[prio].last();
    }

    public void rem_obj(int prio) {
        prqueue[prio].rem_cur();
    }
} // end of Rqueue
