package psimjava;

/**
 * PsimJ Java package for OO simulation
 * <P>Object-oriented simulation package, process interaction approach.
 * <P> (C) J. M. Garrido. June 2000, 2003, 2004
 *
 * Internal class used by class Res.
 * Simple resource queue class for Psim-J
 */
class Rsqueue {
    // private String qname;             // title for queue
    private int q_max; // Maximum number of objects in queue
    private int q_count; // current number of objects
    private Qobject head; // head of queue
    private Qobject tail; // tail of queue
    private Qobject cur_obj;
    //
    public Rsqueue(int max_ob) { // constructor
        // qname = a_name;
        q_max = max_ob;
        q_count = 0;
        head = null;
        tail = null;
        cur_obj = null;
    }

    // overloaded constructor
    public Rsqueue() { // constructor
        // qname = a_name;
        q_max = 2500; // default capacity of queue
        q_count = 0;
        head = null;
        tail = null;
        cur_obj = null;
    }

    public synchronized int length() { // get current number of objects in queue
        return q_count;
    }

    public synchronized boolean full() {
        return (q_max == q_count);
    }

    public synchronized boolean empty() {
        return (q_count == 0);
    }

    public synchronized void into(Object p, int nr, int nreq) {
        // insert p at tail of queue
        if (q_max == q_count) { // is queue full ?
            Pobject.process_error(Pobject.E_PUTFULL);
        }
        Qobject lobj = new Qobject(p, nr, nreq);
        if (q_count == 0) {
            head = lobj;
            tail = lobj;
        }
        else { // place new lobj after tail
            tail.next = lobj;
            lobj.prev = tail;
            tail = lobj;
        }
        q_count++;
    }

    public synchronized void back(Object p, int nr, int nreq) {
        // insert back at the head of the queue
        if (q_max == q_count) { // is queue full ?
            Pobject.process_error(Pobject.E_PUTFULL);
        }
        Qobject lobj = new Qobject(p, nr, nreq);
        if (q_count == 0) {
            head = lobj;
            tail = lobj;
        }
        else {
            head.prev = lobj;
            lobj.next = head;
            head = lobj;
        }
    }

    public synchronized Qobject out() {
        // dequeue an object from (at head) queue
        Qobject p;
        if (q_count == 0) { // is queue empty?
            Pobject.process_error(Pobject.E_GETEMPTY);
        }
        p = head;
        if (q_count == 1) {
            tail = null;
            head = null;
        }
        else {
            head = head.next; // new tail is prev lobj
            head.prev = null;
        }
        q_count--;
        return p;
    }

    public synchronized Qobject last() {
        // remove last object enqueued (from tail)
        Qobject p;
        if (q_count == 0) { // is queue empty?
            Pobject.process_error(Pobject.E_GETEMPTY);
        }
        p = tail;
        if (q_count == 1) {
            tail = null;
            head = null;
        }
        else {
            tail = tail.prev;
            tail.next = null;
        }
        q_count--;
        return p;
    }

    public synchronized void remov(Object pr) {
        // Find and remove object pr from queue
        boolean found = false;
        boolean siga = true; // continue
        Qobject qq = head;
        if (q_count == 0) { // is queue empty?
            Pobject.process_error(Pobject.E_GETEMPTY);
        }
        while ( (found == false) && siga) {
            if (qq.myobj == pr) {
                found = true;
            }
            else {
                if (qq == tail) {
                    siga = false;
                }
                else {
                    qq = qq.next;
                }
            }
        }
        if (q_count == 1) {
            tail = null;
            head = null;
            q_count = 0;
        }
        else {
            qq.prev.next = qq.next; // unlink myobj
            qq.next.prev = qq.prev;
            q_count--;
        }
    }

    public Qobject show_first() {
        // return reference to first object without removing it
        Qobject oo;
        cur_obj = head;
        oo = head;
        return oo;
    }

    public Qobject show_next() {
        // return reference to next object without removing it
        Qobject oo;
        if (cur_obj != null) {
            cur_obj = cur_obj.next;
            oo = cur_obj;
        }
        else {
            oo = null;
        }
        return oo;
    }

    public Qobject show_cur() {
        // show current object
        Qobject oo;
        oo = cur_obj;
        return oo;
    }

    public void rem_cur() {
        // remove current object
        Qobject qq;
        Qobject p = cur_obj;
        if (p == null) {
            // Pobject.process_error(Pobject.E_GETEMPTY);
            return;
        }
        q_count--;
        if (p == head) {
            head = p.next;
            cur_obj = head;
            return;
        }
        if (p != tail) { // object not first nor last
            qq = p.prev;
            qq.next = p.next;
            p.next.prev = qq;
            cur_obj = qq;
            return;
        }
        else { // last object
            qq = p.prev;
            qq.next = null;
            cur_obj = qq;
            tail = qq;
            return;
        }
    }
} // end of Squeue
