package psimjava;

/**
 * Top level class for scheduling processes (implemented as Java threads).
 * <P>PsimJ package for process interaction approach to Object-Oriented
 * discrete-event simulation.
 * (C) Jose M. Garrido, June 2000, August 2003, Oct 2004
 */
public class Sched
    extends Pobject {
    static final int IDLE = 1;
    static final int RUNNING = 2;
    static final int TERMINATED = 3;
    private double s_time; // scheduled time for the process
    private int s_state; // IDLE, RUNNING, TERMINATED
    //
    // ------------------------------------------------------------------
    protected Sched() {
        super(Pobject.TASK);
        s_state = IDLE;
        s_time = 0.0;
    } // end Sched constructor

    //
    //
    void insert(double d) {
        Sched pthis = (Sched) Thread.currentThread();
        StaticSync.insert(pthis, d);
    }

    // ---------------------------------------------------------------
    /**
     * Get current value of simulation clock
     * @return double Returns simulation time.
     */
    public double get_clock() {
        return StaticSync.get_clock();
    }

    //
    /**
     * Resets the simulation clock to time zero.
     * Can only be called after terminating all Psim processes
     */
    void reset_clock() {
        StaticSync.reset_clock();
    }

    //
    void removerc() {
        Process proc = (Process)this;
        StaticSync.removerc(proc);
    }

    //
    public double rdtime() {
        return s_time;
    }

    //
    void set_time(double ltime) {
        s_time = ltime;
    }

    //
    /**
     * Get the state of an object (RUNNING, IDLE, TERMINATED)
     */
    public int rdstate() {
        return s_state;
    }

    public void set_state(int lstate) {
        s_state = lstate;
    }

    //
    void cancel(double res) {
        Process proc = (Process)this;
        if (s_state == RUNNING) {
            StaticSync.removerc(proc);
        }
        s_state = TERMINATED;
        s_time = res;
    }

    //
    double result() {
        Process p;
        Thread pthis = Thread.currentThread();
        // System.out.println("result (Sched)");
        p = (Process) StaticSync.this_process();
        if (p != (Pobject) pthis) {
            process_error(Pobject.E_RESULT);
        }
        if (s_state != TERMINATED) {
            p.deactivate(p);
        }
        // System.out.println("Exiting result (Sched)");
        return s_time;
    }
} // end Sched
