package psimjava;

import java.awt.*;
import javax.swing.*;
import java.awt.event.*;

/**
 * <p> Class that represents a Parameter that is one of a finite set</p>
 *
 * <p>Copyright: Copyright (c) 2005</p>
 *
 * @author Rich Schlesinger
 * @version 1.0
 */
public class SelectionParameter extends Parameter implements ActionListener {
    private String[] options;
    private String selection;
    ////////////////////////////////////////////////////////////////////////////
    /**
     * Selection Parameters have a name, a set of possible values, and a default
     * @param name String
     * @param options String[]
     * @param selection String
     */
    public SelectionParameter(String name, String[] options, String selection) {
        super(name);
        this.options = options;
        this.selection = selection;
    }

    ////////////////////////////////////////////////////////////////////////////
    /**
     * Return current selection
     * @return String
     */
    public String getValue() {
        return selection;
    }
    ////////////////////////////////////////////////////////////////////////////
    /** Used by base class to get current value
     * @return String
     */

    protected String getStringValue() {
        return selection;
    }
    ////////////////////////////////////////////////////////////////////////////
    /**
     * Used by base class to build selection portion of GUI. Uses radio buttons
     * @return JComponent
     */
    protected JComponent buildSelector() {
        Box tBox = Box.createVerticalBox();
        tBox.setBorder(BorderFactory.createLineBorder(Color.black));
        ButtonGroup b = new ButtonGroup();
        // Create a radio button for each option in the list
        for (String option : options) {
            JRadioButton button = new JRadioButton(option);
            button.addActionListener(this);
            if (option.equals(selection)) {
                button.setSelected(true);
            }
            // Add to ButtonGroup and the Box
            b.add(button);
            tBox.add(button);
          }
        return tBox;
    }

    ////////////////////////////////////////////////////////////////////////////
    /**
     * Called by AWT when the selection changes
     * @param e ActionEvent
     */
    public void actionPerformed(ActionEvent e) {
        // User clicked a button; save it
        selection = e.getActionCommand();
        displaySelection(selection);
    }
}
