   package psimjava;

   import java.util.Calendar;

/**
 * Class used by modeler (user) to set up and start a simulation run.
 * <P>An object of this class must be created in the main method of a
 * simulation model.
 * <P> PsimJ Java package for process interaction to object-oriented simulation
 * (C) Jose M. Garrido, June 2000, August 2003, 2004
 */
    public class Simulation {
      public static Res_chain pt_res = new Res_chain();
      public static Bin_chain pt_bin = new Bin_chain();
      private Process pthis;
      private String project_name;
    /**
     * Contructor to initialize simulation.
     * @param sname, title for the simulation project
     */
       public Simulation(String sname) {
         Calendar cal = Calendar.getInstance();
         int month = cal.get(Calendar.MONTH) + 1;
         int day = cal.get(Calendar.DAY_OF_MONTH);
         int year = cal.get(Calendar.YEAR);
         int hour = cal.get(Calendar.HOUR_OF_DAY);
         int min = cal.get(Calendar.MINUTE);
         System.out.println("\nPsim/Java project: " + sname);
         System.out.println("Simulation date: " + month + "/" + day + "/" + year
                           + " " + hour + ":" + min);
         project_name = sname;
      }
   
    //
    /**
     * Start a simulation run with the specified simulation period.
     * @param sim_p Simulation period.
     */
       public void start_sim(double sim_p) {
         double tt; // for latest scheduled time
         Process proc;
         Res resobj;
         Bin binobj;
         Process.set_simul_p(sim_p); // set simulation period
         pthis = (Process) StaticSync.this_process();
        // System.out.println("Application thread: " + pthis.getName()+
        //   " "+pthis.get_name()+
        //   " \n Started: "+StaticSync.get_started()+" created: "+ StaticSync.get_created() );
        //
         while (StaticSync.get_started() < StaticSync.get_created()) {
            Thread.yield();
            //   try {
            //      Thread.sleep(2); }
            //   catch (InterruptedException x) { }
         }
        //
         pthis.delay(sim_p);
        // while (StaticSync.get_clock() < sim_p)
         Thread.yield();
        //
        //
        // Any suspended processes left?  // JNM 10-27-07
         while (StaticSync.get_numSusp() > 0) {
            proc = StaticSync.get_susp(); // get a suspended process
            proc.delay(0.0);  // reactivate suspended process
         }	 
        // Is there a need to adjust time?    07-20-04
         tt = StaticSync.get_schtime();
         if (tt > sim_p) {
            tt = tt - sim_p;
            pthis.delay(tt);
         }
        
         
         //
         System.out.println("Simulation closing at: " + StaticSync.get_clock());
         if (StaticSync.get_clock() >= sim_p) {
            StaticSync.closing();
         }
        //
        // Reschedule current process
         Thread.yield();
         if (StaticSync.get_clock() >= sim_p) {
            // report on usage and waiting for resources
            while (pt_res.current != null) {
                // System.out.println("Report of Res object");
               resobj = (Res) pt_res.get_seq();
               resobj.report();
            }
            while (pt_bin.current != null) {
                // System.out.println("Report of Bin object");
               binobj = (Bin) pt_bin.get_seq();
               binobj.report();
            }
            // System.out.println("\n ---------------------------------------------------");
            System.out.println("\nEnd Simulation of " + project_name +
                               ", clock: " + StaticSync.get_clock());
         }
        // pthis.terminate(pthis);
      }
    //
    /**
     * restore simulation so that it terminates at expiration
     * of the simulation period
     *
        public void restore_sim() {
     pthis.interrupt();    // interrupt the main thread
        }
     */
   }
