   package psimjava;

/**
 * PsimJ Java package for Object Oriented Simulation
 * This class is part of the PsimJ simulation engine.
 * <P>It includes the method that schedules the active objects according
 * to their time stamps.
 * <P> Top level class for scheduling processes.
 * <P>Psim package in Java for process interaction approach to discrete-event
 * simulation.
 * (C) Jose M. Garrido, June 2000, August 2003, 2004
 */
    public class StaticSync
    extends Object {
      private static Pobject p_running = null; // currently executing process
      private static double s_clock = 0.0; // simulation clock
      private static double schtime = 0.0; // latest scheduled time
      private static Sched s_runchain = null; // event list
      private static Squeue sus_procs; // list of suspended processes
      private static Squeue term_procs; // list of terminated processes
      private static boolean first_proc = true;
      public static Process pfirst; // first process
      private static Object susobj;
      private static int started_procs = 0;
      private static int created_procs = 0;
    //
       // get number of suspended processes
       public static int get_numSusp() {
   	   return sus_procs.length();
   	 }
   	 // remove and return a suspended process from list
   	 public static Process get_susp() {
   	   return get_sus();
   	 }
   	 
       public static double get_schtime() {
        // System.out.println("schtime: " + schtime);
         return schtime;
      }
   
    //
       public static void incr_started() {
         started_procs++;
      }
   
    //
       public static void incr_created() {
         created_procs++;
      }
   
    //
       public static int get_started() {
         return started_procs;
      }
   
    //
       public static int get_created() {
         return created_procs;
      }
   
    //
    // place process p at front of sus_procs list
    // invoked by deactivate
       public static synchronized void place_sus(Process p) {
        // System.out.println("StaticSync:place_sus "+p.get_name());
         sus_procs.into(p);
      }
   
       public static boolean first_process() {
         return first_proc;
      }
   
       public static void set_first() {
         first_proc = false;
         susobj = new Object();
         sus_procs = new Squeue("Susp queue");
         term_procs = new Squeue("Terminated procs");
      }
   
       public static void reset_first() {
         first_proc = true;
      }
   
    //
    // place process p at tail of term_procs list
       public static synchronized void place_term(Process p) {
        // System.out.println("place_term proc: "+p.get_name());
         p = null;
        // term_procs.into(p);
      }
   
    //
    // Get the process at front of the sus_procs listgh    // invoked by schedule when the run chain is empty
       private static synchronized Process get_sus() {
         Process p;
         if (sus_procs.empty() == true) {
            return null;
         }
         p = (Process) sus_procs.out();
         return p;
      }
   
    //
    // Get and remove the process at front of the term_procs list
    // invoked by closing
       private static synchronized Process get_term() {
         Process p;
         if (term_procs.empty() == true) {
            return null;
         }
         p = (Process) term_procs.out();
        // System.out.println("get_term proc: "+p.get_name());
         return p;
      }
   
    //
    // Remove process proc from the sus_procs list
    // invoked from reactivate
       private static synchronized void remove_sus(Process proc) {
        // System.out.println ("remove_sus "+ proc.get_name());
         sus_procs.remov(proc);
        // System.out.println("Exiting remove_sus");
      } // end remove_sus
   
    //
       public static synchronized void jjstart_th() {
        // Internal method called by run()
        //
         Process pthis;
         pthis = (Process) Thread.currentThread();
         if (p_running == null) { // first process?
            // System.out.println ("First proc in run chain: "+pthis.getName() +
            //   " " + pthis.get_name());
            p_running = (Pobject) pthis; //
         }
         else {
            // System.out.println("Inserting " + pthis.getName());
            insert(pthis, 0.0); // insert to front of run chain
            try {
               pthis.susobj.suswait();
            }
                catch (InterruptedException x) {
                // ignore
               }
         }
         pthis.setPriority(4);
        //System.out.println ("Completed start_th " + pthis.getName()+ " " +
        //     pthis.get_name());
        // p.setName (t_name);       // set name of thread
      } // end of start_th
   
    //
   
       public static Pobject this_process() {
         return p_running;
      }
   
    //
       public static synchronized void running(Process p) {
         p_running = p;
      }
   
    //
       public static double get_clock() {
         return s_clock;
      }
   
    //
       public static void reset_clock() {
         s_clock = 0.0;
        // System.out.println("Simulation clock reset to 0.0");
      }
   
    // ------------------------------------------------------------------
       public static synchronized void schedule() {
        // simulation clock
         Sched p;
        // Sched pp;
         Process run_proc; // the new running process
         double tt;
        // pp = (Sched) Thread.currentThread();  // current thread
        // Process pthis = (Process) pp;
         Sched ppp = (Sched) p_running;
         Process ppthis = (Process) p_running;
         run_proc = (Process) ppp;
        // System.out.println ("Schedule: "+" running: " + ppthis.get_name());
         show_rchain();
        //if (pp != p_running) {
        //    System.out.println ("Schedule yield running: " + ppp.getName());
        //    Thread.yield();
        //}
         p = s_runchain;
        //while ( p.rdstate() == Sched.TERMINATED && s_runchain != null)
        //   p = s_runchain;
        // remove process from runchain
         if (p != null) {
            s_runchain = (Sched) p.o_next; // remove first object
            p.o_next = null; // from run_chain
            run_proc = (Process) p;
            // System.out.println( "schedule: removed " + p.getName() + " " +
            //       run_proc.get_name() + " from run_chain");
         }
         else {
            run_proc = (Process) ppp; // p_running
            // System.out.println("Run chain empty, p_running: "+ppp.getName() +
            //       "  " + run_proc.get_name());
            if (ppthis != null) { // has this process already canceled itself?
                // System.out.println("Terminating " + ppthis.get_name());
               ppthis.terminate();
            }
            // System.out.println("Exiting schedule with: "+pp.getName());
            return;
         }
        //   }
        //
         if (p.rdstate() == Sched.IDLE && p == pfirst) {
            p.set_state(Sched.RUNNING);
         }
        //
         tt = p.rdtime();
        // System.out.println("Schedule: Time for " + run_proc.get_name() + " is: " + tt +
        //   " clock is: " + s_clock);
         if (tt != s_clock) {
            if (tt < s_clock) {
               Pobject.process_error(Pobject.E_SCHTIME);
            }
            else {
               s_clock = tt; // update the simulation clock
            }
         }
        // System.out.println("Schedule: Clock updated");
        // if ( p != pp )
         p_running = p;
         run_proc = (Process) p;
        //
        // System.out.println ("Schedule: Process " +
        //     run_proc.get_name() + " now the running process");
         show_rchain();
        // System.out.println ("Simulation clock set to: " + s_clock+ " of "
        //      + p.getName() );
        // purge();    // purge any terminated process(es)
        // System.out.println("Exiting schedule ");
      } // end schedule
   
    // ------------------------------------------------------------
    // This routine is called by the application thread (in Simulation)
    // close all processes: suspending, terminated or in run chain
       public static synchronized void closing() {
         Process proc;
         Sched p;
         Process pthis = (Process) p_running;
        // if (pthis != null)
        //   System.out.println("closing: Running is "+pthis.get_name());
        // show_susp();
         while (sus_procs.empty() != true) {
            proc = get_sus(); // get a suspended process
            // System.out.println("Closing susp " + proc.get_name());
            // make it terminated to test, and notify it
            // proc.set_state(Sched.TERMINATED);
            proc.cancel(0.0);
            proc.set_state(Sched.TERMINATED);
            // proc.susobj.proceed();
            proc = null;
         }
         while (term_procs.empty() != true) {
            proc = get_term();
            // System.out.println("Closing: terminating " + proc.get_name());
            proc = null;
         }
         p = s_runchain;
         while (p != null) { // there are processes in the run chain
            proc = (Process) p;
            // System.out.println("Closing: proc in runchain " + proc.get_name());
            s_runchain = (Sched) p.o_next;
            p.o_next = null;
            p = s_runchain;
            proc = null;
         }
      }
   
    //
    // purge terminated processes
       public static synchronized void purge() {
         Process proc;
         while (term_procs.empty() != true) {
            proc = get_term();
            System.out.println("purge: flushing " + proc.get_name());
            proc = null;
         }
        // Thread.yield();
      }
   
    //
       public static synchronized void removerc(Process proc) {
        // remove current process from run chain and make it IDLE
         Sched pthis = (Sched) proc;
        // System.out.println ("Remove "+ proc.get_name()+" from run chain");
         Sched p, pp;
        // show_rchain();
         p = s_runchain;
         if (p != null) {
            if (p == pthis) {
                // System.out.println("removerc: found " + proc.get_name());
               s_runchain = (Sched) p.o_next;
            }
            // look in the whole list
            else { // traverse list searching for pthis
               while (p.o_next != null) {
                  pp = (Sched) p.o_next;
                  if (pp == pthis) {
                        // System.out.println("removerc: found " + proc.get_name());
                     p.o_next = pp.o_next;
                  }
                  else {
                     p = pp;
                  }
               } // end while
            } // end
            pthis.set_state(Sched.IDLE);
            pthis.o_next = null;
         }
         else {
            pthis.set_state(Sched.IDLE);
         }
        // System.out.println("Exiting remove");
      } // end remove
   
    //
       public static synchronized void show_rchain() {
         Process mproc;
         Sched p;
         p = s_runchain;
        // System.out.println("Show runchain");
         while (p != null) {
            mproc = (Process) p;
            // System.out.println("show_rchain: "+mproc.get_name()+ " " + p.rdtime());
            p = (Sched) p.o_next;
         }
        // System.out.println("Exiting show_rchain: ");
      }
   
    //
    //
       public static synchronized void show_susp() {
         Process mproc;
         int pcount;
         pcount = sus_procs.length();
        // System.out.println("Number of suspended procs: "+ pcount);
         while (pcount > 0) {
            mproc = (Process) sus_procs.out();
            // System.out.println("show susp: "+mproc.get_name());
            sus_procs.into(mproc);
            pcount--;
         }
      
        // System.out.println("Exiting show susp: ");
      }
   
    //
       public static synchronized void setclock(double t) {
         if (s_clock > 0.0) {
            Pobject.process_error(Pobject.E_SETCLOCK);
         }
         s_clock = t;
      }
   
    //
    // -------------------------------------------------------------------
       public static synchronized void insert(Sched pthis, double d) {
        // schedule this process to run in d time units
         Sched p, pp;
         Process ppthis = (Process) pthis;
         Process prun = (Process) p_running;
        //    if (prun != null)
        // System.out.println ("Insert: "+ppthis.get_name()+" using "+ d+
        //   " running: " + prun.get_name());
        // Sched pthis = (Sched) Thread.currentThread();
         show_rchain();
         removerc(ppthis);
         if (pthis.rdstate() == Sched.TERMINATED) {
            return;
         }
         if (d < 0.0) {
            Pobject.process_error(Pobject.E_NEGTIME);
         }
      
         double tt = s_clock + d;
        //
        // for latest scheduled time 07-20-04
         if (tt > schtime) {
            schtime = tt;
         }
        //
        // System.out.println ("(StaticSync:insert) using "+ d + " with: " +
        //    ppthis.get_name() + " scheduled at: "+ tt);
         pthis.set_time(tt);
        // System.out.println ("insert: state of " + ppthis.get_name() +
        //            " is: " +
        //          pthis.rdstate());
        /*
               switch (pthis.rdstate()) {
               case Sched.TERMINATED:
               Pobject.process_error (Pobject.E_RESTERM);
               // return;
               break;
               case Sched.IDLE:
               // System.out.println(ppthis.get_name()+ " is IDLE");
               break;
               case Sched.RUNNING:
               //if( (Pobject) pthis != p_running)
               //  Pobject.process_error(Pobject.E_TASKDEL);
               }
         */
         pthis.set_state(Sched.RUNNING);
        // if (pthis.o_next != null)  Pobject.process_error(Pobject.E_RESOBJ);
        // add process pthis (current process) to run chain
         p = s_runchain; // first process in run chain
         if (p != null) { // chain not empty
            if (tt < p.rdtime()) {
                // add current process to front of run chain
                // System.out.println ("Insert: Adding " + ppthis.get_name()+" to front of run chain");
               pthis.o_next = (Pobject) s_runchain;
               s_runchain = (Sched) pthis;
            }
            else {
                // System.out.println ("Insert: Adding "+ ppthis.get_name()+
                //     " to run chain");
               while (p.o_next != null) { // not the end of chain
                    // System.out.println ("Insert, scanning: "+p.getName());
                  pp = (Sched) p.o_next;
                  if (tt < pp.rdtime()) { // insert here if time is 'before'
                     pthis.o_next = pp;
                     p.o_next = (Sched) pthis;
                        // System.out.println(pthis.getName() +" goes before " + pp.getName());
                        // System.out.println ("Exiting insert with "+pthis.getName());
                     return;
                  }
                  else {
                     p = pp;
                  }
               } // end while
               p.o_next = (Sched) pthis;
                // System.out.println(ppthis.get_name()+" goes after "+p.getName());
            }
         }
         else { // run chain is empty
            // System.out.println ("Run chain now not empty");
            s_runchain = (Sched) pthis;
         }
         show_rchain();
        // System.out.println ("Exiting insert " + ppthis.get_name());
      } // end insert
   
    // -------------------------------------------------------------------
    //
       public static synchronized void reactivate(Process p) {
        // a process places a process back in the run list
        // Thread pthis = (Pobject) Thread.currentThread();
        // System.out.println("reactivate "+ p.get_name());
        // System.out.println("reactivate pthis: "+pthis.getName());
        //if ( p_running !=  (Pobject) pthis)
        //    Thread.yield();     // yield to another process
        //else
        // if (! p.idle() )
        //     System.out.println("** reactivate: Process " + p.get_name() +
        //         "is not idle");
        // show_susp();
        // remove_sus(p);
         insert(p, 0.0);
        // show_susp();
        // p.susobj.proceed();
        // System.out.println("Exiting reactivate " + p.get_name());
      }
   
    //
       public static void deactivate(Process p) {
        /* If the process executing this function is deactivating itself,
         then it must call the scheduler.
         */
         boolean to_susp = false;
         Thread pthis = (Pobject) Thread.currentThread();
         Pobject ppp = (Pobject) p_running;
         Process prunn = (Process) p_running;
        // System.out.println("deactivate " + p.get_name()+
        //           " p_running: "+prunn.get_name());
        // while ((Pobject) StaticSync.this_process() !=
        //                             (Pobject) Thread.currentThread()) {
        //System.out.println("deactivate: p_running: "+
        //   StaticSync.this_process().getName()+
        //   " pthis: "+ pthis.getName() + " yield");
        //   Thread.yield();     // yield to another process
        // }
        //
         if (p.rdstate() != Sched.RUNNING) { // is this process active?
            System.out.println("** deactivate: " + p.get_name() +
                               " is not in running" +
                               " state, cur state: " + p.rdstate());
            return; // cannot deactivate p
         }
        //
        /*
         while (prunn.rdstate() == Sched.TERMINATED || prunn.rdstate() ==Sched.IDLE) {
           prunn = (Process) p_running;
           Thread.yield();
               }
         */
         synchronized (susobj) {
            // place_sus(p);      // place in list of suspended processes
            // is this process the current active process?
            if (p != prunn) {
                // System.out.println("deactivate: "+p.get_name() + " not the running proc");
               removerc(p); // remove from run chain and make IDLE
            }
            else {
               p.set_state(Sched.IDLE);
               schedule();
               prunn = (Process) p_running;
                // System.out.println("deactivate: " + p.get_name()+
                //    " notifying new running "+ prunn.get_name());
                // place_sus(p);            // place in list of suspended processes
                // show_susp();
               to_susp = true;
               prunn.proceed(); // notify new running process
            }
         } // end of sync block
        // in case the running process has changed
         prunn = (Process) p_running; // most recent running process
         if (to_susp == true && p_running != p) {
            // System.out.println("deactivate: " + p.get_name() +
            //      " into wait state, running: "+prunn.get_name());
            try {
               p.susobj.set_flag(true);
               p.susobj.suswait();
            }
                catch (InterruptedException x) {
                  x.printStackTrace();
               }
         } // end if
        //
        //
        // System.out.println("Exiting deactivate of: "+p.get_name());
      } // end deactivate
    //
   } // end of class
