package psimjava;

import java.util.Random;
import java.util.Calendar;

/**
 * Random number generator, Uniform distribution
 * <P>PsimJ Java Object-oriented simulation package, process interaction approach.
 * (C) Jose M. Garrido, June 2000, August 2003, 2004
 */
public class Urand implements RandomGenerator {
    // Uniform distribution
    private long l, h;
    private double fl, fh;
    private Random ugen;
    /**
     * Constructor for the random number generator with Uniform dist.
     * @param low long, low bound for the range of values
     * @param high long, upper bound for the range of values
     * @param useed long, the seed to set the random stream.
     */
    public Urand(long low, long high, long useed) {
        ugen = new Random(useed);
        l = low;
        h = high;
    }

    /**
     * Constructor for random number generator with uniform dist
     * using values of type double.
     * @param flow double, low bound for the range of values
     * @param fhigh double, upper bound for the range of values
     * @param useed long, seed ofr the Random nUmber Generator
     */
    public Urand(double flow, double fhigh, long useed) {
        ugen = new Random(useed);
        fl = flow;
        fh = fhigh;
    }

    /**
     * Constructor for random number generator with uniform dist
     * using values of type long.
     * @param low long
     * @param high long
     */
    public Urand(long low, long high) {
        long defseed; // default seed
        Calendar cal = Calendar.getInstance();
        defseed = cal.get(Calendar.SECOND);
        defseed += cal.get(Calendar.MILLISECOND);
        ugen = new Random(defseed);
        l = low;
        h = high;
    }

    /**
     * Constructor for random number generator with uniform dist
     * using values of type double.
     * @param flow double, low bound for the range of values
     * @param fhigh double, upper bound for the range of values
     */

    public Urand(double flow, double fhigh) {
        long defseed; // default seed
        Calendar cal = Calendar.getInstance();
        defseed = cal.get(Calendar.SECOND);
        defseed += cal.get(Calendar.MILLISECOND);
        ugen = new Random(defseed);
        fl = flow;
        fh = fhigh;
    }

    /**
     * This method generates a random number
     * @return long a value that represents a random number.
     */
    public long draw() {
        long y;
        if (l > h) {
            System.out.println("Illegal values to Urand \n");
            Pobject.process_error(Pobject.E_SIMARG);
            return 0;
        }
        else {
            long intervsize = (h - l + 1);
            y = ugen.nextLong();
            // System.out.println( " ugen.nextLong(): " + y);
            long ranoffset = (Math.abs(y) % intervsize);
            return (l + ranoffset);
        }
    }

    /**
     * This method generates a random number
     * @return double a value that represents a random number.
     */
    public double fdraw() {
        if (fl > fh) {
            System.out.println("Illegal values to Urand \n");
            Pobject.process_error(Pobject.E_SIMARG);
            return 0.0;
        }
        else {
            return (ugen.nextDouble() * (fh - fl) + fl);
        }
    }
} // end Urand
