package psimjava;

/**
 * <P>PsimJ Java Object-oriented simulation package, process interaction approach.
 * (C) Jose M. Garrido, June 2000, August 2003, 2004
 * Class Waitq manipulates the cooperation of processes.
 * <P>It allows for one of the processes (master) to dominate and the other to
 * be treated as a slave process, held in the
 * slave queue during the period of cooperation. Objects of this
 * class are used as a synchronization mechanism. The queues are
 * constructed by priorities.
 */

public class Waitq {
    private String title;
    private Pqueue slaveq; // queue of slave processes
    private Pqueue masterq; // queue of master processes
    //
    //
    /**
     * Initializes a Waitq object with its mane and the maximum priority to use
     * @param name title for the object created
     * @param maxprio maximum priority to use
     */
    public Waitq(String name, int maxprio) { // constructor
        title = new String(name);
        if (maxprio <= Pobject.MAX_PRIORITIES) {
            slaveq = new Pqueue(name, maxprio);
            masterq = new Pqueue(name, maxprio);
            // System.out.println(title+ " created");
        }
        else {
            Pobject.process_error(Pobject.E_WTQMPR);
        }
    }

    //
    /**
     * Returns the length (number of processes) in the slave queue
     */
    public int length() { // length of slave queue
        return slaveq.length();
    }

    //
    /**
     * Returns the length (number of processes) in the master queue
     */
    public int lengthm() { // length of masterq
        return masterq.length();
    }

    //
    /**
     * Places current process in the slave queue and passivates (suspends) waiting
     * for a process in the master queue.
     */
    public void qwait() {
        Process ps = (Process) StaticSync.this_process();
        Process lp; // local process
        int masterl; // master queue length

        // System.out.println(title + ":qwait with  " + ps.get_name() );
        masterl = masterq.length();
        // System.out.println(title+":qwait: master length " + masterl);
        if (masterl > 0) { // length of master queue
            lp = (Process) masterq.out(); // highest priority process
            // System.out.println(title+":qwait: from master q (reactivating) "+lp.get_name());
            StaticSync.reactivate(lp); // reactivate
        }
        // System.out.println(title+":qwait: deactivates " + ps.get_name()+ " enqueues");
        slaveq.into(ps); // enqueue ps
        StaticSync.deactivate(ps); // suspend this process
    }

    //
    //
    /**
     * This function returns a slave process
     * from the slave queue. If slave queue is empty, this function passivates
     * the current (master) process.
     */
    public Process coopt() {
        Process ps = (Process) StaticSync.this_process();
        Process lps;
        // int slavel;
        // slavel = slaveq.length();
        boolean wait_cond;
        wait_cond = false; // not coopted yet
        // System.out.println(title+":coopt: "+ps.get_name()+ " slave length "
        //     + slaveq.length());
        while (!wait_cond) {
            wait_cond = bcoopt();
        }
        // if (slavel > 0) {
        lps = (Process) slaveq.out();
        // System.out.println(title+":coopt: process from slave queue: " + lps.get_name());
        return lps; // process from the slave queue
        // }
        /* else {
         // System.out.println(title+":coopt: deactivating " + ps.get_name());
         masterq.into(ps);        // enqueue into master queue
         StaticSync.deactivate(ps); // passivate
         return null;             // process from call to coopt
               }  */
    }

    //
    private boolean bcoopt() {
        Process ps = (Process) StaticSync.this_process();
        Process lps;
        int slavel;
        slavel = slaveq.length();
        // System.out.println(title+":coopt: "+ps.get_name()+ " slave length " + slavel);
        if (slavel > 0) {
            // lps = (Process) slaveq.out();
            //System.out.println(title+":coopt: process from slave queue: " + lps.get_name());
            // return lps;    // process from the slave queue
            return true;
        }
        else {
            // System.out.println(title+":coopt: deactivating master " + ps.get_name());
            masterq.into(ps); // enqueue into master queue
            StaticSync.deactivate(ps); // passivate
            // return null;             // process from call to coopt
            return false;
        }
    }

    //
    /*
        Example of calls to wait, coopt and ccoopt

         waitq qq;                  declare waitq object
        1) example for wait
        qq.wait();
        ......
        ---------------------------------------------------------------
        2) example for coopt
        Process ppp;
        ppp = null;
        ppp = qq.coopt();    // slave process slp
        reactivate(ppp);            // reactivate slp
        ......
     */
    //
} // end of class Waitq
