   package psimjava.gui;

   import javax.swing.*;
   import javax.swing.border.*;
   import java.awt.*;
   import java.awt.event.*;
   import java.io.*;
   import java.util.*;
   import psimjava.*;


    public class PsimFrame extends JFrame
   {
      private BorderLayout frameLayout = new BorderLayout();
      private JPanel framePanel = new JPanel();
      private BorderLayout panelLayout = new BorderLayout();
      private JLabel statusBar = new JLabel();
      private JToolBar toolBar = new JToolBar();
      private JMenuBar menuBar = new JMenuBar();
      private PsimTabbedPane tab = new PsimTabbedPane();
      private JProgressBar jpBar;
      private Action actionRun;
      private Action actionResults;
      private JFileChooser chooser;
   
   //Results Dialog
      private ResultsDialog results;
   
   //Thread to manage ProgressBar
      ProgressThread runner;
   
      SimPanel simPanel;
      private String title = "PsimJ Model";
      private boolean isFirstRun = true;
   
       public PsimFrame(SimPanel sim, String title)
      {
         super();
         try
         {
            this.title = title;
            simPanel = sim;
            setupMenuAndToolbar();
            jbInit();
            chooser = new JFileChooser();
            chooser.setCurrentDirectory(new File("."));
            results = new ResultsDialog(false);
         
         }
             catch (Exception e)
            {
               e.printStackTrace();
            }
      }
   
       private void jbInit() throws Exception
      {
         getContentPane().setLayout(frameLayout);
         setTitle(title);
         setSize(500, 500);
         setJMenuBar(menuBar);
      
         statusBar.setText("");
         getContentPane().add(statusBar, BorderLayout.SOUTH);
         getContentPane().add(toolBar, BorderLayout.NORTH);
         getContentPane().add(framePanel, BorderLayout.CENTER);
      
         framePanel.setLayout(panelLayout);
         framePanel.add(tab, BorderLayout.SOUTH);
         jpBar = new JProgressBar();
         jpBar.setStringPainted(true);
         framePanel.add(jpBar, BorderLayout.CENTER);
         framePanel.add(simPanel, BorderLayout.NORTH);
         pack();
         setFrameLocation();
      }//jbInit()
   
       private void setFrameLocation()
      {
         Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
         Dimension frameSize = this.getSize();
         if (frameSize.height > screenSize.height)
         {
            frameSize.height = screenSize.height;
         }
         if (frameSize.width > screenSize.width)
         {
            frameSize.width = screenSize.width;
         }
         this.setLocation((screenSize.width - frameSize.width)/2,
                                    (screenSize.height - frameSize.height)/2);
         this.addWindowListener(
                new WindowAdapter() {
                   public void windowClosing(WindowEvent e){
                     System.exit(0);
                  }
               });
         this.setVisible(true);
      
      }
   
   
       private void setupMenuAndToolbar()
      {
         menuBar = new JMenuBar();
         JMenu menuFile = new JMenu("File");
         menuFile.setMnemonic('f');
      
         ImageIcon iconSave = new ImageIcon(
                    psimjava.gui.PsimFrame.class.getResource("closefile.gif"));
         Action actionSave = 
             new AbstractAction("Save...", iconSave){
                public void actionPerformed(ActionEvent e){
                  fileSave_ActionPerformed(e);
               }
            };
      
         JMenuItem item = menuFile.add(actionSave);
         item.setMnemonic('s');
         item.setIcon(null);
      
         menuFile.addSeparator();
      
         Action actionExit = 
             new AbstractAction("Exit"){
                public void actionPerformed(ActionEvent e){
                  fileExit_ActionPerformed(e);
               }
            };
      
         item = menuFile.add(actionExit);
         item.setMnemonic('x');
      
         menuBar.add(menuFile);
      
         JMenu menuTools = new JMenu("Tools");
         menuTools.setMnemonic('t');
      
         actionRun = 
             new AbstractAction("Run"){
                public void actionPerformed(ActionEvent e){
                  toolsRun_ActionPerformed(e);
               }
            };
      
         item = menuTools.add(actionRun);
         item.setMnemonic('r');
      
         menuTools.addSeparator();
      
         actionResults = 
             new AbstractAction("Results"){
                public void actionPerformed(ActionEvent e){
                  toolsResults_ActionPerformed(e);
               }
            };
      
         item = menuTools.add(actionResults);
         item.setMnemonic('u');
      
         Action actionGraph = 
             new AbstractAction("Graph"){
                public void actionPerformed(ActionEvent e){
                  toolsGraph_ActionPerformed(e);
               }
            };
      
         item = menuTools.add(actionGraph);
         item.setMnemonic('g');
      
         menuBar.add(menuTools);
      
         JMenu menuHelp = new JMenu("Help");
         menuHelp.setMnemonic('h');
      
         ImageIcon iconHelp = new ImageIcon(
                        psimjava.gui.PsimFrame.class.getResource("help.gif"));
         Action actionHelp = 
             new AbstractAction("Help", iconHelp){
                public void actionPerformed(ActionEvent e){
                  helpHelp_ActionPerformed(e);
               }
            };
      
         item = menuHelp.add(actionHelp);
         item.setMnemonic('h');
         item.setIcon(null);
      
         Action actionAbout = 
             new AbstractAction("About"){
                public void actionPerformed(ActionEvent e){
                  helpAbout_ActionPerformed(e);
               }
            };
      
         menuHelp.addSeparator();
         item = menuHelp.add(actionAbout);
      
         menuBar.add(menuHelp);
      
      //setup toolbar using action objects
         toolBar = new JToolBar();
         JButton buttonSave = toolBar.add(actionSave);
         buttonSave.setText("");
         buttonSave.setToolTipText("Save File");
         buttonSave.setMargin(new Insets(0,0,0,0));
         JButton buttonRun = toolBar.add(actionRun);
         buttonRun.setToolTipText("Run Simulation");
      
         JButton buttonHelp = toolBar.add(actionHelp);
         buttonHelp.setText("");
         buttonHelp.setToolTipText("Help");
         buttonHelp.setMargin(new Insets(0,0,0,0));
      }
   
       public SimPanel getSimPanel()
      {
         return simPanel;
      }
   
       public void setSimPanel(SimPanel sim)
      {
         simPanel = sim;
      }
   
       public OutputStream getOutputStream()
      {
         return tab.getOutputStream();
      }
   
       public void addObserver(Observer obsv)
      {
         tab.addObserver(obsv);
      }
   
       void fileSave_ActionPerformed(ActionEvent e)
      {
         if(chooser.showSaveDialog(this) != JFileChooser.APPROVE_OPTION)
            return;
         else
         {
            File file = chooser.getSelectedFile();
            try{
               FileWriter fw = new FileWriter(file);
               fw.write(tab.getText());
               fw.close();
            }
                catch(IOException ioe)
               {
                  ioe.printStackTrace();
               }
         }
      
      }
   
       void fileExit_ActionPerformed(ActionEvent e)
      {
         results.dispose();
         this.dispose();
         System.exit(0);
      }
   
       void helpAbout_ActionPerformed(ActionEvent e)
      {
      
      }
   
       void helpHelp_ActionPerformed(ActionEvent e)
      {
      
      }
   
       void toolsRun_ActionPerformed(ActionEvent e)
      {
         actionRun.setEnabled(false);
         if(!isFirstRun)
         {
            simPanel.setNull();
            StaticSync.reset_clock();
         }
      
         if(simPanel.setValues())
         {
            if(!isFirstRun)
            {
               tab.createTab();
            //simPanel.setNull();
            // StaticSync.reset_clock();
            }
            else
            {
               isFirstRun = false;
            //StaticSync.reset_clock();
            }
            simPanel.setOutputStream(getOutputStream());
            jpBar.setStringPainted(true);
            simPanel.runSim();
            runner = new ProgressThread();
         
         // StaticSync.reset_clock();
            runner.start();
         }//if
         else
            actionRun.setEnabled(true);
      }
   
       void toolsResults_ActionPerformed(ActionEvent e)
      {
         results.setVisible(true);
      
      }
   
       void toolsGraph_ActionPerformed(ActionEvent e)
      {}
   
       class ProgressThread extends Thread
      {
          private ProgressThread()
         {
            super();
         }
      
          public void run()
         {
            jpBar.setMaximum((int)simPanel.getSimPeriod());
            jpBar.setValue(0);
         
            while(StaticSync.get_clock() < jpBar.getMaximum())
            {
               Runnable runIt = 
                   new Runnable()
                  {
                      public void run()
                     {
                        jpBar.setValue((int)StaticSync.get_clock());
                     }
                  };//anonymous inner class
            
               SwingUtilities.invokeLater(runIt);
               try{
                  Thread.sleep(150);
               }
                   catch(Exception e)
                  {}
            }//while
            Runnable runIt2 = 
                new Runnable()
               {
                   public void run()
                  {
                     jpBar.setValue((int)StaticSync.get_clock());
                  }
               };//anonymous inner class make sure 100%
            SwingUtilities.invokeLater(runIt2);
            actionRun.setEnabled(true);
            while (results == null)
               Thread.yield();
          
            SimResults r = simPanel.getResults();
            r.setTitle("Results "+title);      
         // System.out.println("r: "+ r);
         // System.out.println("results (ProgressThtead): "+results);
            results.addResults(r);
            results.repaint();
            results.setVisible(true);
         
         }//run()
      }//ProgressThread
   }//PsimFrame

